#include <stdio.h>
#include <string.h>
#include <byteswap.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <signal.h>
#include <sqlite3.h>

#include <arpa/inet.h>
#include "../common/conversion.h"
#include "../common/iface.h"
#include "../common/parser.h"
#include "../common/db.h"

int nat_snat_add( int argc, char ** argv );
int nat_dnat_add( int argc, char ** argv );
int group_add( int argc, char **argv );
int client_group( int argc, char ** argv );
int client_add( int argc, char ** argv );
int client_list( int argc, char ** argv );
void usage( int argc, char ** argv );
void usage_group( int argc, char ** argv );
void usage_client( int argc, char ** argv );
void usage_nat( int argc, char ** argv );
void usage_natacl( int argc, char ** argv );
//int natacl_group_add( int argc, char ** argv );
int group_policy_add( int argc, char ** argv );
int group_del( int argc, char **argv );
int group_list( int argc, char **argv );
int group_policy_list( int argc, char ** argv );
int group_policy_del( int argc, char ** argv );
int client_policy_add( int argc, char ** argv );
int client_policy_del( int argc, char ** argv );
int client_policy_list( int argc, char ** argv );
void broadcast_update_client( uint64_t mac );
int reload_group( char * group );




int get_natacl_dhcp_pid()
{
	FILE * fp;
	char buffer[255];
	int value=0;
	
	if((fp = fopen("/var/NatACL/NatACL_dhcp.pid","r"))){
		fread( buffer, 255, 1, fp );
	
		fclose(fp);
	
		value = atoi( buffer );
	}	
	
	return value;
}
int get_natacl_web_pid()
{
	FILE * fp;
	char buffer[255];
	int value=0;
	
	if((fp = fopen("/var/NatACL/NatACL_web.pid","r"))){
		fread( buffer, 255, 1, fp );

		fclose(fp);
	
		value = atoi( buffer );
	}	
	return value;
}
int reload_dhcp()
{
	int pid = get_natacl_dhcp_pid()	;
	if( pid ){
		printf("Sending signal to NatACL_dhcp PID: %d\n", pid );
		fflush(stdout);
		
		kill( pid, 10 );
	}	
	pid = get_natacl_web_pid()	;
	if( pid ){
		printf("Sending signal to NatACL_web PID: %d\n", pid );
		fflush(stdout);
		
		kill( pid, 10 );
	}
	return 1;
}
int main( int argc, char ** argv )
{
	
	if( argc<2 ){
		usage( argc, argv);
		exit( 0 );	
	}
	if( strcmp( argv[1],"group") == 0 ){
		if( argc < 3 ){
			usage_group( argc, argv );	
			exit ( 0 );
		}
		if( strcmp( argv[2], "add" ) == 0 ){
			group_add( argc, argv );
		}			
		else if( strcmp( argv[2], "del" ) == 0 ){
			group_del( argc, argv );
		}			
		else if( strcmp( argv[2], "list" ) == 0 ){
			group_list( argc, argv );
		}			

		else if( strcmp( argv[2], "policy_add" ) == 0 ){
			group_policy_add( argc, argv );
		}			
		else if( strcmp( argv[2], "policy_list" ) == 0 ){
			group_policy_list( argc, argv );
		}			
		else if( strcmp( argv[2], "policy_del" ) == 0 ){
			group_policy_del( argc, argv );
		} else {
			usage_group( argc, argv );	
		}


	}		
	else if( strcmp( argv[1],"client") == 0 ){
		if( argc < 3 ){
			usage_client( argc, argv );	
			exit ( 0 );
		}
		if( strcmp( argv[2], "add" ) == 0 ){
			client_add( argc, argv );
		}
		else if( strcmp( argv[2], "list" ) == 0 ){
			client_list( argc, argv );
		}
		else if( strcmp( argv[2], "group" ) == 0 ){
			client_group( argc, argv );
		}
		else if( strcmp( argv[2], "policy_add" ) == 0 ){
			client_policy_add( argc, argv );
		}			
		else if( strcmp( argv[2], "policy_list" ) == 0 ){
			client_policy_list( argc, argv );
		}			
		else if( strcmp( argv[2], "policy_del" ) == 0 ){
			client_policy_del( argc, argv );
		} else {
			usage_client( argc, argv );			
		}
	} else {
			usage( argc, argv);
	}
	return 0;
}
int get_file_size(char *filename)
{
    struct stat st;

    if (stat(filename, (struct stat *) &st) != 0) {
	return -1;
    }
    return st.st_size;
}

int group_policy_add( int argc, char ** argv )
{
		char buffer[255];
		char option[255];
		struct s_db_group s_group;
		sqlite3 * db;
		
		if( argc <6 ){
			printf("Usage: %s group policy_add <group_name> [<type> <type_options>]\n", argv[0]);
			printf("	type: group <group_name>\n");
			printf("	type: snat <source_ip_address>\n");
			printf("	type: dnat <destination_ip_address> <destination_port>\n");
			printf("	type: natacl <multiple_sessions 1/0> <expire_options> <authentication_options>\n");
			printf("	type: exec <ONCE 1/0> '<command>'\n");
			printf("\n");
			printf("		<expire_options>\n");
			printf("			EXPIRE_PING=[ping_interval]\n");
			printf("			EXPIRE_POPUP\n");
			printf("			EXPIRE_TIME=[time_interval]\n");
			
			printf("\n");
			printf("		<authentication_options>\n");
			printf("			auth_unix.so\n");
			printf("			auth_pop3.so|SERVER=10.10.0.9\n");



			fflush(stdout);
			exit ( 0 );	
		}
		db = init_db();

		memset( &s_group, 0x00, sizeof( struct s_db_group ));

		get_group_by_name( db, argv[3], &s_group );

		if( !s_group.bu_group_id ){
			printf("Could not find group: %s\n", argv[3] );
			exit ( 0 );
		}

		if( strcmp( argv[4],"snat" ) == 0 ){
			if( argc < 6 ){
				printf("Usage: %s group policy_add <group_name> snat <source_nat_ip_address>\n", argv[0] );
				fflush(stdout); 
				exit( 0 );	
			}
			db_insert_group_policy( db, s_group.bu_group_id, argv[4], "",  argv[5], 0 );

			reload_group( argv[3] );
//			reload_dhcp();
		}
		else if( strcmp( argv[4],"dnat" ) == 0 ){
			if( argc < 7 ){
				printf("Usage: %s group policy_add <group_name> dnat <destination_nat_ip_address> <destination port>\n", argv[0] );
				fflush(stdout); 
				exit( 0 );	
			}
			db_insert_group_policy( db, s_group.bu_group_id, argv[4], "", argv[5], atoi( argv[6]) );
//			reload_dhcp();
			reload_group( argv[3] );
		}
		else if( strcmp( argv[4],"group" ) == 0 ){
			if( argc < 6 ){
				printf("Usage: %s group policy_add <group_name> group <group_name>\n", argv[0] );
				fflush(stdout); 
				exit( 0 );	
			}
			if( have_group_network_access( db, s_group.bu_group_id, argv[5]  ) ){			
				printf("Group policy already exist\n");
			} else {
				db_insert_group_policy( db, s_group.bu_group_id, argv[4], argv[5], "", 0 );				
			}			
			get_group_by_name( db, argv[5], &s_group );
			
			if( !have_group_network_access( db, s_group.bu_group_id, argv[3]  ) ){			
				db_insert_group_policy( db, s_group.bu_group_id, argv[4], argv[3], "", 0 );
			}
			reload_group( argv[3] );
			reload_group( argv[5] );

//			reload_dhcp();
		}
		else if( strcmp( argv[4],"exec" ) == 0 ){
			if( argc < 7 ){
				printf("Usage: %s group policy_add <group> exec [once 1/0] '<command>'\n", argv[0] );
				fflush(stdout); 
				exit( 0 );	
			}
			if( argv[5][0] != '1' && argv[5][0] !='0' ){
				printf("Usage: %s client policy_add <group> exec [<only once 1/0>] '<command>'\n", argv[0] );
				fflush(stdout); 
				exit( 0 );	
			}
			snprintf( buffer, 255, "ONCE=%s|COMMAND=%s",  argv[5], argv[6]  );
		
		
			db_insert_group_policy( db, s_group.bu_group_id, argv[4], buffer, "", 0 );
			reload_group( argv[3] );
			
//			reload_dhcp();
		}
		else if( strcmp( argv[4],"natacl" ) == 0 ){		
			if( argc < 7 ){
				printf("	type: natacl <multiple_sessions 1/0> <expire_options> <authentication_options>\n");
				printf("\n");
				printf("		<expire_options>\n");
				printf("			EXPIRE_PING=[ping_interval]\n");
				printf("			EXPIRE_POPUP\n");
				printf("			EXPIRE_TIME=[time_interval]\n");
				
				printf("\n");
				printf("		<authentication_options>\n");
				printf("			auth_unix.so\n");
				printf("			auth_pop3.so|SERVER=10.10.0.9\n");
				exit(0);
			}
			if( !(strncmp( argv[6], "EXPIRE_PING=", strlen("EXPIRE_PING=")) ==0 ||
				strcmp(  argv[6], "EXPIRE_POPUP" ) ==0 ||
				strncmp( argv[6], "EXPIRE_TIME=",strlen("EXPIRE_TIME=") )== 0) ){
					
				printf("Invalid Expire value [%s]\n", argv[6]);	
				exit  ( 0 );
			}
			
		    snprintf( buffer, 255, "MULTIPLE_SESSION=%d|EXPIRE_METHOD=%s|NAME=%s", atoi( argv[5] ), argv[6], argv[7]  );
		
		    get_option( "NAME", buffer, option, 255 );
		
		    if( get_file_size( option ) ==-1 ){
				perror( option );
				exit  ( 0 );
		    }
		
			db_insert_group_policy( db, s_group.bu_group_id, argv[4], buffer, NULL, 0 );
//			reload_dhcp();
			reload_group( argv[3] );

		} else {
			printf("Invalid Argument: %s\n", argv[4] );
			fflush(stdout);	
		}

		sqlite3_close(db);

		return 1;
}
int group_policy_list( int argc, char ** argv )
{
	sqlite3 * db;
	if( argc <4 ){
		printf("Usage: %s group policy_list <group_name>\n",argv[0] );
		fflush(stdout);
		exit ( 0 );	
	}
	
	db = init_db();

	db_policy_list( db, argv[3] );
	
	sqlite3_close(db );
		
		return 1;
}
int group_policy_del( int argc, char ** argv )
{
	sqlite3 * db;
		if( argc <4 ){
			printf("Usage: %s group policy_del <id>>\n",argv[0] );
			fflush(stdout);
			exit ( 0 );	
		}
		db = init_db();

		db_group_policy_del( db, atoi(argv[3]) );

		sqlite3_close(db);
		
//		reload_dhcp();
		reload_group( argv[3] );

		return 1;
}

int client_policy_add( int argc, char ** argv )
{
	sqlite3 * db;
		char buffer[255];
		char option[255];
//		struct s_db_hdr s_db;
		unsigned long long v;

		if( argc <6 ){
			printf("Usage: %s client policy_add <mac_address> [<type> <type_options>]\n", argv[0]);
			printf("	type: snat <source_ip_address>\n");
			printf("	type: dnat <destination_ip_address> <destination_port>\n");
			printf("	type: natacl <multiple_sessions 1/0> <expire_options> <authentication_options>\n");
			printf("	type: exec <ONCE 1/0> '<command>'\n");
			printf("\n");
			printf("		<expire_options>\n");
			printf("			EXPIRE_PING=[ping_interval]\n");
			printf("			EXPIRE_POPUP\n");
			printf("			EXPIRE_TIME=[time_interval]\n");
			
			printf("\n");
			printf("		<authentication_options>\n");
			printf("			auth_unix.so\n");
			printf("			auth_pop3.so|SERVER=10.10.0.9\n");



			fflush(stdout);
			exit ( 0 );	
		}

		v = mac_str2int( (unsigned char *)argv[3] );
		db = init_db();

		if( !v ){
			printf("Invalid MAC ADDRESS: %s\n", argv[3] );
			fflush(stdout);
			exit(0);	
		}
		if( strcmp( argv[4],"snat" ) == 0 ){
			if( argc < 6 ){
				printf("Usage: %s client policy_add <mac_address> snat <source_nat_ip_address>\n", argv[0] );
				fflush(stdout); 
				exit( 0 );	
			}
			db_insert_client_policy( db, v, argv[4], "",  argv[5], 0 );
			broadcast_update_client( v );

//			reload_dhcp();
		}
		else if( strcmp( argv[4],"group" ) == 0 ){
			if( argc < 7 ){
				printf("Usage: %s client policy_add <group_name> group <group_name>\n", argv[0] );
				fflush(stdout); 
				exit( 0 );	
			}
			db_insert_client_policy( db, v, argv[4], "", argv[5], 0 );
//			reload_dhcp();
			broadcast_update_client( v );

		}
		else if( strcmp( argv[4],"dnat" ) == 0 ){
			if( argc < 7 ){
				printf("Usage: %s client policy_add <mac_address> dnat <destination_nat_ip_address> <destination port>\n", argv[0] );
				fflush(stdout); 
				exit( 0 );	
			}
			db_insert_client_policy( db, v, argv[4], "", argv[5], atoi( argv[6]) );
//			reload_dhcp();
			broadcast_update_client( v );

		}
		else if( strcmp( argv[4],"exec" ) == 0 ){
			if( argc < 7 ){
				printf("Usage: %s client policy_add <mac_address> exec <only once 1/0> '<command>'\n", argv[0] );
				fflush(stdout); 
				exit( 0 );	
			}
			if( argv[5][0] != '1' && argv[5][0] !='0' ){
				printf("Usage: %s client policy_add <mac_address> exec [<only once 1/0>] '<command>'\n", argv[0] );
				fflush(stdout); 
				exit( 0 );	
			}
			snprintf( buffer, 255, "ONCE=%s|COMMAND=%s",  argv[5], argv[6]  );
		
			db_insert_client_policy( db, v, argv[4], buffer, "", 0 );
//			reload_dhcp();
			broadcast_update_client( v );
		}
		else if( strcmp( argv[4],"natacl" ) == 0 ){		
			if( argc < 7 ){
				printf("	type: natacl <multiple_sessions 1/0> <expire_options> <authentication_options>\n");
				printf("\n");
				printf("		<expire_options>\n");
				printf("			EXPIRE_PING=[ping_interval]\n");
				printf("			EXPIRE_POPUP\n");
				printf("			EXPIRE_TIME=[time_interval]\n");
				
				printf("\n");
				printf("		<authentication_options>\n");
				printf("			auth_unix.so\n");
				printf("			auth_pop3.so|SERVER=10.10.0.9\n");
				exit(0);
			}
			if( !(strncmp( argv[6], "EXPIRE_PING=", strlen("EXPIRE_PING=")) ==0 ||
				strcmp(  argv[6], "EXPIRE_POPUP" ) ==0 ||
				strncmp( argv[6], "EXPIRE_TIME=",strlen("EXPIRE_TIME=") )== 0) ){
					
				printf("Invalid Expire value [%s]\n", argv[6]);	
				exit  ( 0 );
			}



		    snprintf( buffer, 255, "MULTIPLE_SESSION=%d|EXPIRE_METHOD=%s|NAME=%s", atoi( argv[5] ), argv[6], argv[7]  );
		
		    get_option( "NAME", buffer, option, 255 );
		
		    if( get_file_size( option ) ==-1 ){
				perror( option );
				exit  ( 0 );
		    }
		
			db_insert_client_policy( db, v, argv[4], buffer, NULL, 0 );
//			reload_dhcp();
			broadcast_update_client( v );
		} else {
			printf("Invalid Argument: %s\n", argv[4] );
			fflush(stdout);	
		}
		sqlite3_close(db);

		return 1;
}
int client_policy_list( int argc, char ** argv )
{
	sqlite3 * db;
		unsigned long long v;
		
		if( argc< 4 ){
			printf("Usage: %s client policy_list <mac_address>\n",argv[0] );
			fflush(stdout);
			
			exit(0);
		}
		
		v = mac_str2int( (unsigned char *)argv[3] );
		db = init_db();

		db_client_policy_list( db, v );
		sqlite3_close(db);
		
		return 1;
}
int client_policy_del( int argc, char ** argv )
{
	sqlite3 * db;
		if( argc <4 ){
			printf("Usage: %s client policy_del <id>>\n",argv[0] );
			fflush(stdout);
			exit ( 0 );	
		}
		db = init_db();
		db_client_policy_del( db, atoi(argv[3]) );
		sqlite3_close(db);
		
		reload_dhcp();


		return 1;
}


int group_add( int argc, char **argv )
{
	sqlite3 * db;
    struct s_iface_hdr iface;

	if( argc <7 ){
		printf("WARNING: Each group must have it's own interface/alias\n");
		printf("Ex: eth1:1, eth1:2, eth1:3, eth2, eth3\n");
		
		printf("Usage: %s group add <name> <interface> <dns_address> <wins_address> <domain>\n",argv[0] );
		fflush(stdout);

		exit ( 0 );	
	}


		
	if( get_iface_by_name( &iface, argv[4] ) == 0 ){
	    printf("Unable to find interface [%s]\n", argv[4] );
	    exit( 0 );
	}

//	int db_insert_group(char *name, uint32_t router_address, uint32_t broadcast_address, char * dns_address, char *wins_address, char *domain)
//    printf("BROAD: %ld\n", (unsigned long )iface.broadcast );
//    fflush(stdout);
	db = init_db();
	
	db_insert_group( db, argv[3], iface.addr, iface.broadcast, iface.netmask, argv[5], argv[6], argv[7]);			
	sqlite3_close(db);
//	reload_dhcp();

	reload_group( argv[3] );

	return 1;
}

int group_del( int argc, char **argv )
{
	struct s_db_group s_group;
		memset( &s_group, 0x00, sizeof( struct s_db_group ));
	
	sqlite3 * db;
	
	if( argc <4 ){
		printf("Usage: %s group del <name>\n",argv[0] );
		fflush(stdout);

		exit ( 0 );	
	}

	db = init_db();
	memset( &s_group, 0x00, sizeof( struct s_db_group ));

	get_group_by_name( db, argv[3], &s_group );

	if( !s_group.bu_group_id ){
		printf("Could not find group: %s\n", argv[3] );
		exit ( 0 );
	}

	db_del_group( db, s_group.bu_group_id );
//	reload_dhcp();

	reload_group( argv[3] );

	return 1;
}
int group_list( int argc, char **argv )
{
	sqlite3 * db;
//        struct s_iface_hdr iface;
//	struct s_db_hdr s_db;
//	memset( &s_db, 0x00, sizeof( struct s_db_hdr ));

	db = init_db();


	db_group_list( db );

	return 1;
}

uint32_t get_free_address( sqlite3 * db, struct s_db_hdr * s_db, char * group_name )
{
	struct s_db_group s_group;
	uint32_t i;
	
		memset( &s_group, 0x00, sizeof( struct s_db_group ));

    if (!get_group_by_name(db, group_name , &s_group)) {
        printf("ERROR: There is no group called 'unknown', This group is required. create it using \"NatACL_config group add\"\n");
        exit(0);
    }

	for( i=s_group.router_address + 10;i<s_group.broadcast_address;i++ ){
		if( !search_client_by_ip( s_db->s_clients, i ) ){
			return i;	
		}		
	}
	return 0;
}

int client_group( int argc, char ** argv )
{
	sqlite3 * db;
	long long v=0;
	uint32_t addr;
	struct s_db_group s_group;
	
	
	struct s_db_hdr s_db;
	
	memset( &s_db, 0x00, sizeof( struct s_db_hdr ));
		memset( &s_group, 0x00, sizeof( struct s_db_group ));
	
	if( argc <4 ){
		printf("Usage: %s client group <mac_address> <group>\n",argv[0] );
		fflush(stdout);
		exit ( 0 );	
	}
	
//	v = bswap_64( strtoll( argv[3], NULL, 16 ) ) >> 16;
	v = mac_str2int( (unsigned char *)argv[3] );


	db = init_db();

	get_group_by_name( db, argv[4], &s_group );

	if( !s_group.bu_group_id ){
		printf("Could not find group: %s\n", argv[4] );
		exit ( 0 );
	}

	db_get_address( db, &s_db );

	addr = get_free_address( db, &s_db, argv[4] );

	db_change_group( db, v, addr, s_group.bu_group_id );		

    broadcast_update_client( v );

	return 1;
}










int client_add( int argc, char ** argv )
{
	sqlite3 * db;
		long long v=0;
		struct s_db_hdr s_db;
		struct s_db_group s_group;
		uint32_t addr;
		memset( &s_db, 0x00, sizeof( struct s_db_hdr ));
		memset( &s_group, 0x00, sizeof( struct s_db_group ));


		db = init_db();
	
		if( argc <7 ){
			printf("Usage: %s client add <mac_address> <ip_address> <group_name> <name>\n",argv[0] );
			fflush(stdout);
			exit ( 0 );	
		}
		v = mac_str2int( (unsigned char *)argv[3] );

		get_group_by_name( db, argv[5], &s_group );	
	
		if( !s_group.bu_group_id ){
			printf("Could not find group: %s\n", argv[5] );
			exit ( 0 );
		}
		db_get_address( db, &s_db );

		if( !inet_addr(argv[4] )) {
			addr = get_free_address( db, &s_db, argv[5] );
		} else {
			addr= ntohl(inet_addr(argv[4]));
		}

		db_insert_client(db, &s_group, v, argv[6], addr  );

	    broadcast_update_client( v );

		return 1;
}
int client_list( int argc, char ** argv )
{
	sqlite3 * db;
	
	db = init_db();
	
	 db_list_address(db);

		return 1;
}

void usage( int argc, char ** argv )
{
//	if(	argc < 2 ){
		printf("Usage:\n");
		printf("	%s group	- DHCP Groups\n",argv[0]);
		printf("	%s client	- DHCP Clients ( machines )\n",argv[0]);
//		printf("	%s nat	- NAT Configuration\n",argv[0]);
//		printf("	%s natacl	- NatACL Configuration\n",argv[0]);
		
		exit( 0 );
//	}	
}
void usage_group( int argc, char ** argv )
{
	printf("Usage:\n");
	printf(" %s group add <name> <interface> <dns_address> <wins_address> <domain>\n", argv[0] );
	printf("	- Add a group configuration\n");
	printf(" %s group del <name>\n", argv[0] );
	printf("	- Remove a group ( and all clients to that group )\n");
	printf(" %s group list\n", argv[0] );
	printf("	- List groups\n");
	printf("\n");
	printf(" %s group policy_add\n", argv[0] );
	printf(" %s group policy_list <group_name>\n", argv[0] );
	printf(" %s group policy_del <policy_id>\n", argv[0] );
	
}
void usage_client( int argc, char ** argv )
{	
	printf("Usage:\n");

	printf("%s client add  <mac_address> <ip_address> <group_name> <name>\n", argv[0] );
	printf("	bind a mac_address to an ip_address/group\n" );

	printf("%s client group <mac_address> <group>\n", argv[0] );
	printf("	Change a client's group\n" );

	printf("%s client list\n", argv[0] );
	printf("	List clients\n" );

	printf("\n");
	printf(" %s group policy_add\n", argv[0] );
	printf(" %s group policy_list <mac_address>\n", argv[0] );
	printf(" %s group policy_del <policy_id>\n", argv[0] );

}
void usage_natacl( int argc, char ** argv )
{
	printf("Usage:\n");
}
void broadcast_update_client( uint64_t mac )
{
	unsigned char * str_mac;
	char firewall_rule[255];

	str_mac = ( unsigned char * )&mac;

    snprintf(firewall_rule, 255, "iptables -F NatACL_fwd_%02x%02x%02x%02x%02x%02x >/dev/null 2>/dev/null", str_mac[0],str_mac[1],str_mac[2],str_mac[3],str_mac[4],str_mac[5] );
    system(firewall_rule);
    snprintf(firewall_rule, 255, "iptables -t nat -F NatACL_snat_%02x%02x%02x%02x%02x%02x  >/dev/null 2>/dev/null", str_mac[0],str_mac[1],str_mac[2],str_mac[3],str_mac[4],str_mac[5]);
    system(firewall_rule);
    snprintf(firewall_rule, 255, "iptables -t nat -F NatACL_dnat_%02x%02x%02x%02x%02x%02x  >/dev/null 2>/dev/null", str_mac[0],str_mac[1],str_mac[2],str_mac[3],str_mac[4],str_mac[5]);
    system(firewall_rule);
    snprintf(firewall_rule, 255, "iptables -t nat -F NatACL_web_%02x%02x%02x%02x%02x%02x  >/dev/null 2>/dev/null", str_mac[0],str_mac[1],str_mac[2],str_mac[3],str_mac[4],str_mac[5]);
    system(firewall_rule);

    snprintf(firewall_rule, 255, "iptables -D NatACL_fwd -j NatACL_fwd_%02x%02x%02x%02x%02x%02x >/dev/null 2>/dev/null", str_mac[0],str_mac[1],str_mac[2],str_mac[3],str_mac[4],str_mac[5]);
    system(firewall_rule);
    snprintf(firewall_rule, 255, "iptables -t nat -D NatACL_snat -j NatACL_snat_%02x%02x%02x%02x%02x%02x >/dev/null 2>/dev/null",  str_mac[0],str_mac[1],str_mac[2],str_mac[3],str_mac[4],str_mac[5]);
    system(firewall_rule);
    snprintf(firewall_rule, 255, "iptables -t nat -D NatACL_dnat -j NatACL_dnat_%02x%02x%02x%02x%02x%02x >/dev/null 2>/dev/null", str_mac[0],str_mac[1],str_mac[2],str_mac[3],str_mac[4],str_mac[5]);
    system(firewall_rule);
    snprintf(firewall_rule, 255, "iptables -t nat -D NatACL_web  -j NatACL_web_%02x%02x%02x%02x%02x%02x >/dev/null 2>/dev/null", str_mac[0],str_mac[1],str_mac[2],str_mac[3],str_mac[4],str_mac[5]);
    system(firewall_rule);

    snprintf(firewall_rule, 255, "iptables -X NatACL_fwd_%02x%02x%02x%02x%02x%02x >/dev/null 2>/dev/null" , str_mac[0],str_mac[1],str_mac[2],str_mac[3],str_mac[4],str_mac[5]);
    system(firewall_rule);
    snprintf(firewall_rule, 255, "iptables -t nat -X NatACL_snat_%02x%02x%02x%02x%02x%02x  >/dev/null 2>/dev/null", str_mac[0],str_mac[1],str_mac[2],str_mac[3],str_mac[4],str_mac[5]);
    system(firewall_rule);
    snprintf(firewall_rule, 255, "iptables -t nat -X NatACL_dnat_%02x%02x%02x%02x%02x%02x  >/dev/null 2>/dev/null", str_mac[0],str_mac[1],str_mac[2],str_mac[3],str_mac[4],str_mac[5]);
    system(firewall_rule);
    snprintf(firewall_rule, 255, "iptables -t nat -X NatACL_web_%02x%02x%02x%02x%02x%02x  >/dev/null 2>/dev/null", str_mac[0],str_mac[1],str_mac[2],str_mac[3],str_mac[4],str_mac[5]);
    system(firewall_rule);

    snprintf(firewall_rule, 255, "iptables -N NatACL_reload_%02x%02x%02x%02x%02x%02x  >/dev/null 2>/dev/null", str_mac[0],str_mac[1],str_mac[2],str_mac[3],str_mac[4],str_mac[5]);
    system(firewall_rule);
		
}
int reload_group( char * group )
{
	char firewall_rule[255];
	
	snprintf( firewall_rule, 255, "iptables -N NatACL_reload_%s", group );
	system(firewall_rule );	
	return 1;
}
