#define TYPE_SNAT 1
#define TYPE_DNAT 2
#define TYPE_NATACL 3
#define TYPE_EXEC 4
#define TYPE_GROUP 5

#define DB_FILENAME "/var/NatACL/NatACL.v1.db"

struct s_client_hdr {
	uint32_t client_address;
	uint32_t router_address;
	uint32_t broadcast_address;
	uint32_t dns_address;
	uint32_t wins_address;
	uint32_t bu_group_id;	
	uint64_t int_mac_address;
	uint32_t netmask_address;
	unsigned char mac_address[6];
	
	int exec_tv;
	int last_tv;
	int vrfy_tv;
	int lock;
	
	char domain [30];
	char group_name[64];
	char name[128];

	struct s_list * s_group_policy; 
	struct s_list * s_client_policy; 
};
struct s_db_policy {
	uint32_t id;
	uint32_t bu_group_id;
	uint32_t type;
	
	uint32_t snat_address;
	uint32_t dnat_address;
	uint32_t dnat_port;
	int once;
	int exec_tv;
	
	int executed;
	char config[1024];
};
struct s_db_group {
	uint32_t id;
	uint32_t router_address;
	uint32_t netmask_address;
	uint32_t broadcast_address;
	uint32_t dns_address;
	uint32_t wins_address;
	uint32_t bu_group_id;	
	char domain [30];
	char group_name[64];
};
struct s_db_hdr {
	struct s_list * s_clients;
	struct s_list * s_groups;
	struct s_list * s_queue_command;
	pthread_mutex_t mutex_queue_command;         
	pthread_mutex_t mutex_iptables;         

	sqlite3 *db;
	
	int sock_fds[65535];
	int sock_fds_count;
	int sock_fds_flag;

	int init;
};
struct s_exec_hdr {
	char command[4096];
	int  verify;
	
	int type;
	char table[255];
	char chain[255];
};

int select_sourcenat_callback(void *p_data, int num_fields, char **p_fields, char **p_col_names);
int select_destinationnat_callback(void *p_data, int num_fields, char **p_fields, char **p_col_names);
int select_list_address_callback(void *p_data, int num_fields, char **p_fields, char **p_col_names);
int select_groupid_callback(void *p_data, int num_fields, char **p_fields, char **p_col_names);
int select_firewall_group_callback(void *p_data, int num_fields, char **p_fields, char **p_col_names);
struct s_list * db_get_address(sqlite3 *db, struct s_db_hdr * s_db );
int db_client_prop(sqlite3 *db, struct s_client_hdr *s_client, unsigned char *address);
struct s_client_hdr * search_client_by_mac( struct s_list * s_l, uint64_t mac_address );
int get_group_by_name(sqlite3 *db,char *name, struct s_db_group * s_group);
int db_insert_client(sqlite3 *db,struct s_db_group *s_group, uint64_t address, char * name, uint32_t client_address );
struct s_client_hdr * search_client_by_ip( struct s_list * s_l, uint32_t address );
struct s_list * db_get_address_by_mac(sqlite3 *db, struct s_db_hdr * s_db, uint64_t mac_address );
struct s_list * db_get_address(sqlite3 *db, struct s_db_hdr * s_db );
int free_memory( struct s_db_hdr * s_db );
struct s_client_hdr * db_get_single_client_by_address(sqlite3 *db, uint32_t client_address );
struct s_db_policy * db_get_natacl_from_client( struct s_client_hdr * s_client );
int db_free_client( struct s_client_hdr * s_client );
sqlite3 * init_db();




int db_insert_group_policy(sqlite3 * db, int groupid, char * type, char * config, char * address, int port  );
int db_group_policy_del( sqlite3 * db, int id );
int db_insert_client_policy(sqlite3 * db, unsigned long long mac_address, char * type, char * config, char * address, int port  );
int db_client_policy_list( sqlite3 * db, unsigned long long mac_address );
int db_client_policy_del( sqlite3* db, int id );
int db_insert_group(sqlite3 * db, char *name, uint32_t router_address, uint32_t broadcast_address,uint32_t netmask_address, char * dns_address, char *wins_address, char *domain);
int db_del_group( sqlite3 *db, int group_id );
int db_group_list(sqlite3 *db);
int db_change_group(sqlite3 * db, long long v, unsigned long client_address, int group_id);
int db_list_address(sqlite3 *db);
int db_policy_list( sqlite3 *db, char * name );
int db_get_group(sqlite3 *db, struct s_db_hdr * s_db );
int have_group_network_access( sqlite3 *db, int group_id, char * config  );
struct s_db_group * cache_get_group_by_name( struct s_list * s_l, char * name );











