#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include <iptables.h>
#include <libiptc/libiptc.h>

#include <pthread.h>

char *freadline_static(FILE * fp, char *buffer, int size, int *pos)
{
    int i;
    char temp;
    int readed = 0;

    i = 0;
    do {
	if (fread(&temp, 1, 1, fp) <= 0) {
	    break;
	}
	if (temp != 0x0a && temp != 0x0d) {
	    buffer[i++] = temp;
	}
	readed++;
    } while (temp != 0x0a && temp != 0x0d && !feof(fp) && i < size);

    if (pos) {
	*pos = readed;
    }
    buffer[i] = 0x00;

    return buffer;
}

int iptables_pos_detect_forward(FILE * fp)
{
    char buffer[255];
    int pos = 0;

    while (!feof(fp)) {
	freadline_static(fp, buffer, 255, &pos);

	if (strncmp(buffer, "Chain FORWARD", strlen("Chain FORWARD")) == 0) {
	    freadline_static(fp, buffer, 255, &pos);
	    return 1;
	}
    }
    return 0;

}
int iptables_pos_detect_prerouting(FILE * fp)
{
    char buffer[255];
    int pos = 0;

    while (!feof(fp)) {
	freadline_static(fp, buffer, 255, &pos);

	if (strncmp(buffer, "Chain PREROUTING", strlen("Chain PREROUTING")) == 0) {
	    freadline_static(fp, buffer, 255, &pos);
	    return 1;
	}
    }
    return 0;

}
int iptables_pos_detect_chain(FILE * fp, char * name)
{
    char buffer[255];
    char line[255];
    int pos = 0;

    while (!feof(fp)) {
	snprintf( line, 255, "Chain %s", name );
	freadline_static(fp, buffer, 255, &pos);
	
//    printf("Searching : [%s] [%s]\n" , buffer,line );
//    fflush(stdout);
	
	if (strncasecmp(buffer, line, strlen(line)) == 0) {
	    freadline_static(fp, buffer, 255, &pos);
//	    printf("FOUND\n" );
//	    fflush(stdout);
	    return 1;
	}
    }
    return 0;

}

int check_forward(uint64_t mac, uint32_t address)
{
    FILE *fp;
    int pos = 0;
    char buffer[255];
    int result = 0;
    char str_mac[255];
    int i;
    char *s;
    char *saveptr = NULL;
    char *token;
    unsigned char *c_mac;

    char opt[64][255];
    int cursor = 0;
    struct in_addr in;


    c_mac = (unsigned char *) &mac;

    in.s_addr = address;
    snprintf(str_mac, 255, "%02x:%02x:%02x:%02x:%02x:%02x", c_mac[0], c_mac[1], c_mac[2], c_mac[3], c_mac[4], c_mac[5]);

    if (!(fp = popen("iptables -L -n", "r"))) {
	return 0;
    }
    if (!iptables_pos_detect_forward(fp)) {
	pclose(fp);

	return 0;
    }

    while (!feof(fp)) {
	freadline_static(fp, buffer, 255, &pos);
	s = buffer;

	if (strlen(buffer) == 0) {
	    break;
	}
	for (i = 0; i < 64; i++)
	    opt[i][0] = 0x00;

	cursor = 0;
	while ((token = strtok_r(s, " ", &saveptr))) {
	    s = NULL;
	    if (cursor == 64) {
		break;
	    }

	    snprintf(opt[cursor], 255, "%s", token);
	    cursor++;
	}
	if (strcmp(opt[0], "ACCEPT") == 0 && strcmp(opt[1], "all") == 0 && strcmp(opt[3], inet_ntoa(in)) == 0 && strcmp(opt[4], "0.0.0.0/0") == 0 && strcmp(opt[5], "MAC") == 0 && strcasecmp(opt[6], str_mac) == 0) {


//	    printf("HAVE IT *******************\n");
//	    fflush(stdout);
	    result = 1;
	    break;
	}
    }


    pclose(fp);

    return result;
}

int check_forward_natacl(uint64_t mac, uint32_t address, uint32_t router_address)
{
    FILE *fp;
    int pos = 0;
    char buffer[255];
    int result = 0;
    char str_mac[255];
    int i;
    char *s;
    char *saveptr = NULL;
    char *token;
    unsigned char *c_mac;

    char opt[64][255];
    int cursor = 0;
    struct in_addr in;
    struct in_addr ind;


    c_mac = (unsigned char *) &mac;

    in.s_addr = address;
    ind.s_addr = router_address;
    snprintf(str_mac, 255, "%02x:%02x:%02x:%02x:%02x:%02x", c_mac[0], c_mac[1], c_mac[2], c_mac[3], c_mac[4], c_mac[5]);

    if (!(fp = popen("iptables -L -n", "r"))) {
	return 0;
    }
    if (!iptables_pos_detect_forward(fp)) {
	pclose(fp);

	return 0;
    }

    while (!feof(fp)) {
	freadline_static(fp, buffer, 255, &pos);
	s = buffer;

	if (strlen(buffer) == 0) {
	    break;
	}
	for (i = 0; i < 64; i++)
	    opt[i][0] = 0x00;

	cursor = 0;
	while ((token = strtok_r(s, " ", &saveptr))) {
	    s = NULL;
	    if (cursor == 64) {
		break;
	    }

	    snprintf(opt[cursor], 255, "%s", token);
	    
//	    printf("[%d] %s\n", cursor, token );
//	    fflush(stdout);
	    cursor++;
	}
	
	if (strcmp(opt[0], "ACCEPT") == 0 
	&& strcmp(opt[1], "tcp") == 0 
	&& strcmp(opt[3], inet_ntoa(in)) == 0 
	&& strcmp(opt[5], "tcp") == 0 
	&& strcasecmp(opt[6], "dpt:5121") == 0 
	&& strcasecmp(opt[7], "MAC") == 0 
	&& strcasecmp(opt[8], str_mac) == 0) {


	    result = 1;
	    break;
	}
    }


    pclose(fp);

    return result;
}
int chain_create( char * chain_name )
{
    char buffer[255];
    
    snprintf( buffer, 255, "iptables -N %s", chain_name );

    system( buffer );
    return 1;
    
}
int chain_vrfy( char * name, pthread_mutex_t * mutex_iptables  )
{
    iptc_handle_t table;

 	pthread_mutex_lock(mutex_iptables);
 
    table = iptc_init ("filter");
    if( iptc_is_chain( name, table ) ){
		iptc_commit( &table );
 	pthread_mutex_unlock(mutex_iptables);
    	return 1;
    }
	iptc_commit( &table );

 	pthread_mutex_unlock(mutex_iptables);

    return 0;
}
int iptables_chain_flush( char * tablename,  char * chain )
{
    iptc_handle_t table;
    table = iptc_init (tablename );

	iptc_flush_entries( chain, &table );	
	iptc_commit( &table );
	return 1;	
}
int iptables_chain_delete(char * tablename,  char * chain )
{
    iptc_handle_t table;
    table = iptc_init (tablename );

	iptc_delete_chain( chain, &table );	
	iptc_commit( &table );
	return 1;	
}
int iptables_chain_create( char * tablename, char * chain )
{
    iptc_handle_t table;
    table = iptc_init (tablename );

	iptc_create_chain( chain, &table );	
	iptc_commit( &table );
	return 1;	
}
