#define DHCP_OP_BOOTREQUEST 1
#define DHCP_OP_BOOTREPLY   2

#define DHCP_TYPE_DISCOVER 0x01
#define DHCP_TYPE_OFFER 0x02
#define DHCP_TYPE_REQUEST 0x03
#define DHCP_TYPE_ACK 0x05
#define DHCP_TYPE_NAK 0x06
#define DHCP_TYPE_RELEASE 0x07
#define DHCP_TYPE_INFORM 0x08


#define ETH_HW_ADDR_LEN 6
#define IP_ADDR_LEN 4
#define ARP_FRAME_TYPE 0x0806
#define ETHER_HW_TYPE 1
#define IP_PROTO_TYPE 0x0800
#define OP_ARP_REQUEST 2


#define ARPOP_REQUEST   1               /* ARP request.  */
#define ARPOP_REPLY     2               /* ARP reply.  */
#define ARPOP_RREQUEST  3               /* RARP request.  */
#define ARPOP_RREPLY    4               /* RARP reply.  */
#define ARPOP_InREQUEST 8               /* InARP request.  */
#define ARPOP_InREPLY   9               /* InARP reply.  */
#define ARPOP_NAK       10              /* (ATM)ARP NAK.  */

#define FILE_LOG "/var/log/NatACL/NatACL_dhcp.log"

struct arp_packet {
        u_char targ_hw_addr[ETH_HW_ADDR_LEN];
        u_char src_hw_addr[ETH_HW_ADDR_LEN];
        u_short frame_type;
        u_short hw_type;
        u_short prot_type;
        u_char hw_addr_size;
        u_char prot_addr_size;
        u_short op;
        u_char sndr_hw_addr[ETH_HW_ADDR_LEN];
        u_char sndr_ip_addr[IP_ADDR_LEN];
        u_char rcpt_hw_addr[ETH_HW_ADDR_LEN];
        u_char rcpt_ip_addr[IP_ADDR_LEN];
        u_char padding[18];
};

struct s_comm_dhcp {
	char op;
	char htype;
	char hlen;
	char hops;
	
	uint32_t xid;
	
	unsigned short secs;
	unsigned short flags;
	
	uint32_t claddr;
	uint32_t yladdr;
	uint32_t sladdr;
	uint32_t gladdr;
	char chaddr[16];
	char sname[64];
	char file[128];
};

struct s_dhcp_hdr {
	uint32_t id;
	unsigned char address[8];
	char type;
	uint32_t requested_client_address;
	uint32_t requested_router_address;
	uint32_t requested_dns_address;
	uint32_t requested_wins_address;
	uint32_t requested_broadcast_address;
	uint32_t requested_dhcp_address;
	
	uint32_t source_address;
	unsigned response_client_address;
	unsigned response_dhcp_address;
	unsigned response_dns_address;
	unsigned response_router_address;
	unsigned response_broadcast_address;
	unsigned response_wins_address;
	char response_domain[64];
	
};

uint32_t mac_addr_sys ( u_char *addr, char * interface);
int dhcp_send_response(char *interface, struct s_dhcp_hdr *s_dhcp );
int dhcp_bind();
int dhcp_decode_packet(struct s_dhcp_hdr *s_dhcp, char *packet, int packet_size );




int mac_addr_idx ( char * interface);
int dhcp_option_add_message_type( char * buffer, int cursor, int type );
int dhcp_option_add_serverid( char * buffer, int cursor, uint32_t addr );

int dhcp_option_add_leasetime( char * buffer, int cursor, uint32_t time );
int dhcp_option_add_netmask( char * buffer, int cursor, uint32_t netmask );
int dhcp_option_add_renew( char * buffer, int cursor, uint32_t value );
int dhcp_option_add_router( char * buffer, int cursor, uint32_t router );
int dhcp_option_add_dns( char * buffer, int cursor, uint32_t dns );
int dhcp_option_add_domain( char * buffer, int cursor, char * domain );
int dhcp_option_add_broadcast( char * buffer, int cursor, uint32_t broadcast );
int dhcp_option_add_netbiosns( char * buffer, int cursor, uint32_t ip );

int dhcp_send_response_nak(char *interface, struct s_dhcp_hdr *s_dhcp);
int dhcp_option_add_rebind(char *buffer, int cursor, uint32_t value);

