#include<stdio.h>
#include <stdlib.h>
#include <string.h>

#include <netinet/in.h>

#include <unistd.h>

#include <stdio.h>
#include <sqlite3.h>
#include <byteswap.h>
#include <time.h>
#include <signal.h>
#include <arpa/inet.h>
#include <fcntl.h>
#include <net/if.h>
#include <pthread.h>


#include "dhcp.h"
#include "db.h"
#include "../common/iptables.h"
#include "../common/file.h"
#include "../includes/list.h"
#include "../common/iface.h"
#include "../common/parser.h"

#define FILENAME_LOCK "/var/NatACL/dhcp.lock"

char *invert_mac_order(char *source);
void usage(char *argv0);
int handle_policy(struct s_db_hdr *s_db, struct s_client_hdr *s_client, struct s_list *s_l, int check_natacl);
int check_lock();
void init_rules(struct s_db_hdr *s_db, struct s_list *s_clients, struct s_list *s_groups);
int client_add(sqlite3 * db, uint64_t mac_address, char *str, uint32_t client_address, char *name);
int create_pidfile(int pid);
int handle_policy_exec(struct s_db_hdr *s_db, struct s_client_hdr *s_client, struct s_list *s_l, int check_natacl);
int default_network_group(struct s_db_hdr *s_db, struct s_client_hdr *s_client, struct s_list *s_l);
int expire_clients(struct s_db_hdr *s_db);
int delete_dhcp_rule(struct s_db_hdr *s_db, struct s_client_hdr *s_client);
int delete_dhcp_rule_group(struct s_db_hdr *s_db, struct s_db_group *s_group);
int command_add(struct s_db_hdr *s_db, int type, char *tablename, char *command, int verify);
void *command_handler(void *data);




struct s_db_hdr s_db;


uint32_t get_free_address(sqlite3 * db, struct s_db_hdr *s_db, char *group_name);


char *ntoa_address(char *buffer, unsigned long addr)
{
    struct in_addr in;

    in.s_addr = addr;

    snprintf(buffer, 255, "%s", inet_ntoa(in));

    return buffer;
}


void init_rules(struct s_db_hdr *s_db, struct s_list *s_clients, struct s_list *s_groups)
{
    struct s_client_hdr *s_client;
    struct s_db_group *s_group;
    struct s_db_group *s_group2;
    struct s_list_hdr *s_walker;
    struct s_list_hdr *s_walker_group;
    char firewall_rule[255];
    char str_source_netmask_address[64];
    char str_source_router_address[64];
    char str_dest_netmask_address[64];
    char str_dest_router_address[64];


    wlog(FILE_LOG, "[INFO] Flushing iptables");

    snprintf(firewall_rule, 255, "iptables -D FORWARD -j NatACL_fwd        >/dev/null 2>/dev/null");
    command_add(s_db, TYPE_IPTABLES_COMMAND, NULL, firewall_rule, 0);
    snprintf(firewall_rule, 255, "iptables -t nat -D POSTROUTING -j NatACL_snat   >/dev/null 2>/dev/null");
    command_add(s_db, TYPE_IPTABLES_COMMAND, NULL, firewall_rule, 0);
    snprintf(firewall_rule, 255, "iptables -t nat -D PREROUTING -j NatACL_dnat    >/dev/null 2>/dev/null");
    command_add(s_db, TYPE_IPTABLES_COMMAND, NULL, firewall_rule, 0);
    snprintf(firewall_rule, 255, "iptables -t nat -D PREROUTING -j NatACL_web     >/dev/null 2>/dev/null");
    command_add(s_db, TYPE_IPTABLES_COMMAND, NULL, firewall_rule, 0);


    command_add(s_db, TYPE_IPTABLES_FLUSH, "filter", "NatACL_LOGDROP", 0);
    command_add(s_db, TYPE_IPTABLES_FLUSH, "filter", "NatACL_fwd", 0);
    command_add(s_db, TYPE_IPTABLES_FLUSH, "nat", "NatACL_snat", 0);
    command_add(s_db, TYPE_IPTABLES_FLUSH, "nat", "NatACL_dnat", 0);
    command_add(s_db, TYPE_IPTABLES_FLUSH, "nat", "NatACL_web", 0);

    if (s_clients) {
        s_walker = s_clients->s_first;
        while (s_walker) {
            s_client = (struct s_client_hdr *) s_walker->data;

            snprintf(firewall_rule, 255, "NatACL_gfwd_%lu", (unsigned long )s_client->router_address);
            command_add(s_db, TYPE_IPTABLES_FLUSH, "filter", firewall_rule, 0);
            snprintf(firewall_rule, 255, "NatACL_gsnat_%lu", (unsigned long )s_client->router_address);
            command_add(s_db, TYPE_IPTABLES_FLUSH, "nat", firewall_rule, 0);
            snprintf(firewall_rule, 255, "NatACL_gdnat_%lu", (unsigned long )s_client->router_address);
            command_add(s_db, TYPE_IPTABLES_FLUSH, "nat", firewall_rule, 0);
            snprintf(firewall_rule, 255, "NatACL_gweb_%lu", (unsigned long )s_client->router_address);
            command_add(s_db, TYPE_IPTABLES_FLUSH, "nat", firewall_rule, 0);

            snprintf(firewall_rule, 255, "NatACL_fwd_%02x%02x%02x%02x%02x%02x", s_client->mac_address[0], s_client->mac_address[1], s_client->mac_address[2], s_client->mac_address[3], s_client->mac_address[4], s_client->mac_address[5]);
            command_add(s_db, TYPE_IPTABLES_FLUSH, "filter", firewall_rule, 1);
            snprintf(firewall_rule, 255, "NatACL_snat_%02x%02x%02x%02x%02x%02x", s_client->mac_address[0], s_client->mac_address[1], s_client->mac_address[2], s_client->mac_address[3], s_client->mac_address[4], s_client->mac_address[5]);
            command_add(s_db, TYPE_IPTABLES_FLUSH, "nat", firewall_rule, 0);
            snprintf(firewall_rule, 255, "NatACL_dnat_%02x%02x%02x%02x%02x%02x", s_client->mac_address[0], s_client->mac_address[1], s_client->mac_address[2], s_client->mac_address[3], s_client->mac_address[4], s_client->mac_address[5]);
            command_add(s_db, TYPE_IPTABLES_FLUSH, "nat", firewall_rule, 0);
            snprintf(firewall_rule, 255, "NatACL_web_%02x%02x%02x%02x%02x%02x", s_client->mac_address[0], s_client->mac_address[1], s_client->mac_address[2], s_client->mac_address[3], s_client->mac_address[4], s_client->mac_address[5]);
            command_add(s_db, TYPE_IPTABLES_FLUSH, "nat", firewall_rule, 0);


            snprintf(firewall_rule, 255, "NatACL_fwd_%02x%02x%02x%02x%02x%02x", s_client->mac_address[0], s_client->mac_address[1], s_client->mac_address[2], s_client->mac_address[3], s_client->mac_address[4], s_client->mac_address[5]);
            command_add(s_db, TYPE_IPTABLES_REMOVE, "filter", firewall_rule, 1);
            snprintf(firewall_rule, 255, "NatACL_snat_%02x%02x%02x%02x%02x%02x", s_client->mac_address[0], s_client->mac_address[1], s_client->mac_address[2], s_client->mac_address[3], s_client->mac_address[4], s_client->mac_address[5]);
            command_add(s_db, TYPE_IPTABLES_REMOVE, "nat", firewall_rule, 0);
            snprintf(firewall_rule, 255, "NatACL_dnat_%02x%02x%02x%02x%02x%02x", s_client->mac_address[0], s_client->mac_address[1], s_client->mac_address[2], s_client->mac_address[3], s_client->mac_address[4], s_client->mac_address[5]);
            command_add(s_db, TYPE_IPTABLES_REMOVE, "nat", firewall_rule, 0);
            snprintf(firewall_rule, 255, "NatACL_web_%02x%02x%02x%02x%02x%02x", s_client->mac_address[0], s_client->mac_address[1], s_client->mac_address[2], s_client->mac_address[3], s_client->mac_address[4], s_client->mac_address[5]);
            command_add(s_db, TYPE_IPTABLES_REMOVE, "nat", firewall_rule, 0);

            snprintf(firewall_rule, 255, "NatACL_gfwd_%lu", (unsigned long )s_client->router_address);
            command_add(s_db, TYPE_IPTABLES_REMOVE, "filter", firewall_rule, 0);
            snprintf(firewall_rule, 255, "NatACL_gsnat_%lu", (unsigned long )s_client->router_address);
            command_add(s_db, TYPE_IPTABLES_REMOVE, "nat", firewall_rule, 0);
            snprintf(firewall_rule, 255, "NatACL_gdnat_%lu", (unsigned long )s_client->router_address);
            command_add(s_db, TYPE_IPTABLES_REMOVE, "nat", firewall_rule, 0);
            snprintf(firewall_rule, 255, "NatACL_gweb_%lu", (unsigned long )s_client->router_address);
            command_add(s_db, TYPE_IPTABLES_REMOVE, "nat", firewall_rule, 0);

            s_walker = s_walker->s_right;
        }
    }




    command_add(s_db, TYPE_IPTABLES_REMOVE, "filter", "NatACL_LOGDROP", 0);
    command_add(s_db, TYPE_IPTABLES_REMOVE, "filter", "NatACL_fwd", 0);
    command_add(s_db, TYPE_IPTABLES_REMOVE, "nat", "NatACL_snat", 0);
    command_add(s_db, TYPE_IPTABLES_REMOVE, "nat", "NatACL_dnat", 0);
    command_add(s_db, TYPE_IPTABLES_REMOVE, "nat", "NatACL_web", 0);

    command_add(s_db, TYPE_IPTABLES_CREATE, "filter", "NatACL_LOGDROP", 0);
    command_add(s_db, TYPE_IPTABLES_CREATE, "filter", "NatACL_fwd", 0);
    command_add(s_db, TYPE_IPTABLES_CREATE, "nat", "NatACL_snat", 0);
    command_add(s_db, TYPE_IPTABLES_CREATE, "nat", "NatACL_dnat", 0);
    command_add(s_db, TYPE_IPTABLES_CREATE, "nat", "NatACL_web", 0);

    snprintf(firewall_rule, 255, "iptables -A NatACL_LOGDROP -j LOG  >/dev/null 2>/dev/null");
    command_add(s_db, TYPE_IPTABLES_COMMAND, NULL, firewall_rule, 0);
    snprintf(firewall_rule, 255, "iptables -A NatACL_LOGDROP -j DROP  >/dev/null 2>/dev/null");
    command_add(s_db, TYPE_IPTABLES_COMMAND, NULL, firewall_rule, 0);


    snprintf(firewall_rule, 255, "iptables -I FORWARD -j NatACL_fwd");
    command_add(s_db, TYPE_IPTABLES_COMMAND, NULL, firewall_rule, 1);
    snprintf(firewall_rule, 255, "iptables -t nat -I POSTROUTING -j NatACL_snat");
    command_add(s_db, TYPE_IPTABLES_COMMAND, NULL, firewall_rule, 1);
    snprintf(firewall_rule, 255, "iptables -t nat -I PREROUTING -j NatACL_dnat");
    command_add(s_db, TYPE_IPTABLES_COMMAND, NULL, firewall_rule, 0);
    snprintf(firewall_rule, 255, "iptables -t nat -I PREROUTING -j NatACL_web");
    command_add(s_db, TYPE_IPTABLES_COMMAND, NULL, firewall_rule, 0);
    // TO avoid proxy on the intranet */


    if (s_groups) {
        s_walker = s_groups->s_first;
        while (s_walker) {
            s_group = (struct s_db_group *) s_walker->data;

            snprintf(firewall_rule, 255, "NatACL_gfwd_%u", s_group->router_address);
            command_add(s_db, TYPE_IPTABLES_FLUSH, "filter", firewall_rule, 0);
            command_add(s_db, TYPE_IPTABLES_CREATE, "filter", firewall_rule, 0);

            snprintf(firewall_rule, 255, "NatACL_gsnat_%u", s_group->router_address);
            command_add(s_db, TYPE_IPTABLES_FLUSH, "filter", firewall_rule, 0);
            command_add(s_db, TYPE_IPTABLES_CREATE, "nat", firewall_rule, 0);

            snprintf(firewall_rule, 255, "NatACL_gdnat_%u", s_group->router_address);
            command_add(s_db, TYPE_IPTABLES_FLUSH, "filter", firewall_rule, 0);
            command_add(s_db, TYPE_IPTABLES_CREATE, "nat", firewall_rule, 0);

            snprintf(firewall_rule, 255, "NatACL_gweb_%u", s_group->router_address);
            command_add(s_db, TYPE_IPTABLES_FLUSH, "filter", firewall_rule, 0);
            command_add(s_db, TYPE_IPTABLES_CREATE, "nat", firewall_rule, 0);

            snprintf(firewall_rule, 255, "iptables -A NatACL_fwd -s %s/%s -j NatACL_gfwd_%lu", ntoa_address(str_source_router_address, ntohl(s_group->router_address)), ntoa_address(str_source_netmask_address, ntohl(s_group->netmask_address)), (unsigned long) s_group->router_address);
            command_add(s_db, TYPE_IPTABLES_COMMAND, NULL, firewall_rule, 1);
            snprintf(firewall_rule, 255, "iptables -A NatACL_fwd -s %s/%s -j RETURN", ntoa_address(str_source_router_address, ntohl(s_group->router_address)), ntoa_address(str_source_netmask_address, ntohl(s_group->netmask_address)));
            command_add(s_db, TYPE_IPTABLES_COMMAND, NULL, firewall_rule, 1);

            snprintf(firewall_rule, 255, "iptables -t nat -A NatACL_dnat -s %s/%s -j NatACL_gdnat_%lu", ntoa_address(str_source_router_address, ntohl(s_group->router_address)), ntoa_address(str_source_netmask_address, ntohl(s_group->netmask_address)), (unsigned long) s_group->router_address);
            command_add(s_db, TYPE_IPTABLES_COMMAND, NULL, firewall_rule, 1);
            snprintf(firewall_rule, 255, "iptables -t nat -A NatACL_dnat -s %s/%s -j RETURN", ntoa_address(str_source_router_address, ntohl(s_group->router_address)), ntoa_address(str_source_netmask_address, ntohl(s_group->netmask_address)));
            command_add(s_db, TYPE_IPTABLES_COMMAND, NULL, firewall_rule, 1);

            snprintf(firewall_rule, 255, "iptables -t nat -A NatACL_snat -s %s/%s -j NatACL_gsnat_%lu", ntoa_address(str_source_router_address, ntohl(s_group->router_address)), ntoa_address(str_source_netmask_address, ntohl(s_group->netmask_address)), (unsigned long) s_group->router_address);
            command_add(s_db, TYPE_IPTABLES_COMMAND, NULL, firewall_rule, 1);
            snprintf(firewall_rule, 255, "iptables -t nat -A NatACL_snat -s %s/%s -j RETURN", ntoa_address(str_source_router_address, ntohl(s_group->router_address)), ntoa_address(str_source_netmask_address, ntohl(s_group->netmask_address)));
            command_add(s_db, TYPE_IPTABLES_COMMAND, NULL, firewall_rule, 1);

            snprintf(firewall_rule, 255, "iptables -t nat -A NatACL_web -s %s/%s -j NatACL_gweb_%lu", ntoa_address(str_source_router_address, ntohl(s_group->router_address)), ntoa_address(str_source_netmask_address, ntohl(s_group->netmask_address)), (unsigned long) s_group->router_address);
            command_add(s_db, TYPE_IPTABLES_COMMAND, NULL, firewall_rule, 1);
            snprintf(firewall_rule, 255, "iptables -t nat -A NatACL_web -s %s/%s -j RETURN", ntoa_address(str_source_router_address, ntohl(s_group->router_address)), ntoa_address(str_source_netmask_address, ntohl(s_group->netmask_address)));
            command_add(s_db, TYPE_IPTABLES_COMMAND, NULL, firewall_rule, 1);

            s_walker_group = s_groups->s_first;
            while (s_walker_group) {
                s_group2 = (struct s_db_group *) s_walker_group->data;

                if (s_group->router_address != s_group2->router_address) {

//                    snprintf(firewall_rule, 255, "iptables -t nat -I NatACL_gdnat_%lu -s %s/%s -d %s/%s -j ACCEPT >/dev/null 2>/dev/null", (unsigned long) s_group->router_address, ntoa_address(str_source_router_address, ntohl(s_group->router_address)), ntoa_address(str_source_netmask_address, ntohl(s_group->netmask_address)), ntoa_address(str_dest_router_address, ntohl(s_group2->router_address)), ntoa_address(str_dest_netmask_address, ntohl(s_group2->netmask_address)));
                    snprintf(firewall_rule, 255, "iptables -t nat -I NatACL_gdnat_%lu  -d %s/%s -j ACCEPT >/dev/null 2>/dev/null", (unsigned long) s_group->router_address, ntoa_address(str_dest_router_address, ntohl(s_group2->router_address)), ntoa_address(str_dest_netmask_address, ntohl(s_group2->netmask_address)));
                    command_add(s_db, TYPE_IPTABLES_COMMAND, NULL, firewall_rule, 1);
                }
                s_walker_group = s_walker_group->s_right;

            }

            s_walker = s_walker->s_right;
        }
    }


    wlog(FILE_LOG, "[INFO] Flushed");


}
int process_client(struct s_db_hdr *s_db, struct s_client_hdr *s_client)
{
    char firewall_rule[255];
    char str_client_address[64];
    int have_natacl = 0;

    have_natacl = 0;


    if (db_get_natacl_from_client(s_client)) {
        wlog(FILE_LOG, "[INFO] NATACL OPTION FOUND");

        have_natacl = 1;
    }

    wlog(FILE_LOG, "[%02x:%02x:%02x:%02x:%02x:%02x] [RUNNING IPTABLES]", s_client->mac_address[0], s_client->mac_address[1], s_client->mac_address[2], s_client->mac_address[3], s_client->mac_address[4], s_client->mac_address[5]);

    snprintf(firewall_rule, 255, "NatACL_fwd_%02x%02x%02x%02x%02x%02x", s_client->mac_address[0], s_client->mac_address[1], s_client->mac_address[2], s_client->mac_address[3], s_client->mac_address[4], s_client->mac_address[5]);
    command_add(s_db, TYPE_IPTABLES_CREATE, "filter", firewall_rule, 0);
    snprintf(firewall_rule, 255, "NatACL_snat_%02x%02x%02x%02x%02x%02x", s_client->mac_address[0], s_client->mac_address[1], s_client->mac_address[2], s_client->mac_address[3], s_client->mac_address[4], s_client->mac_address[5]);
    command_add(s_db, TYPE_IPTABLES_CREATE, "nat", firewall_rule, 0);
    snprintf(firewall_rule, 255, "NatACL_dnat_%02x%02x%02x%02x%02x%02x", s_client->mac_address[0], s_client->mac_address[1], s_client->mac_address[2], s_client->mac_address[3], s_client->mac_address[4], s_client->mac_address[5]);
    command_add(s_db, TYPE_IPTABLES_CREATE, "nat", firewall_rule, 0);
    snprintf(firewall_rule, 255, "NatACL_web_%02x%02x%02x%02x%02x%02x", s_client->mac_address[0], s_client->mac_address[1], s_client->mac_address[2], s_client->mac_address[3], s_client->mac_address[4], s_client->mac_address[5]);
    command_add(s_db, TYPE_IPTABLES_CREATE, "nat", firewall_rule, 0);

    snprintf(firewall_rule, 255, "iptables -A NatACL_gfwd_%lu  -p all -s %s  -j NatACL_fwd_%02x%02x%02x%02x%02x%02x", (unsigned long) s_client->router_address, ntoa_address(str_client_address, htonl(s_client->client_address)), s_client->mac_address[0], s_client->mac_address[1], s_client->mac_address[2], s_client->mac_address[3], s_client->mac_address[4], s_client->mac_address[5]);
    command_add(s_db, TYPE_IPTABLES_COMMAND, NULL, firewall_rule, 1);
    snprintf(firewall_rule, 255, "iptables -A NatACL_gfwd_%lu  -p all -s %s  -j RETURN", (unsigned long) s_client->router_address, ntoa_address(str_client_address, htonl(s_client->client_address)));
    command_add(s_db, TYPE_IPTABLES_COMMAND, NULL, firewall_rule, 1);

    default_network_group(s_db, s_client, s_db->s_groups);

    snprintf(firewall_rule, 255, "iptables -A NatACL_fwd_%02x%02x%02x%02x%02x%02x -m mac --mac-source %02x:%02x:%02x:%02x:%02x:%02x -j ACCEPT", s_client->mac_address[0], s_client->mac_address[1], s_client->mac_address[2], s_client->mac_address[3], s_client->mac_address[4], s_client->mac_address[5], s_client->mac_address[0], s_client->mac_address[1], s_client->mac_address[2], s_client->mac_address[3], s_client->mac_address[4], s_client->mac_address[5]);
    command_add(s_db, TYPE_IPTABLES_COMMAND, NULL, firewall_rule, 1);



    snprintf(firewall_rule, 255, "iptables -t nat -A NatACL_gsnat_%lu  -s %s  -j NatACL_snat_%02x%02x%02x%02x%02x%02x ", (unsigned long) s_client->router_address, ntoa_address(str_client_address, htonl(s_client->client_address)), s_client->mac_address[0], s_client->mac_address[1], s_client->mac_address[2], s_client->mac_address[3], s_client->mac_address[4], s_client->mac_address[5]);
    command_add(s_db, TYPE_IPTABLES_COMMAND, "nat", firewall_rule, 1);
    snprintf(firewall_rule, 255, "iptables -t nat -A NatACL_gsnat_%lu  -s %s  -j RETURN ", (unsigned long) s_client->router_address, ntoa_address(str_client_address, htonl(s_client->client_address)));
    command_add(s_db, TYPE_IPTABLES_COMMAND, "nat", firewall_rule, 1);
        snprintf(firewall_rule, 255, "iptables -t nat -A NatACL_gdnat_%lu -s %s  -j NatACL_dnat_%02x%02x%02x%02x%02x%02x ", (unsigned long) s_client->router_address, ntoa_address(str_client_address, htonl(s_client->client_address)), s_client->mac_address[0], s_client->mac_address[1], s_client->mac_address[2], s_client->mac_address[3], s_client->mac_address[4], s_client->mac_address[5]);
        command_add(s_db, TYPE_IPTABLES_COMMAND, "nat", firewall_rule, 1);

        snprintf(firewall_rule, 255, "iptables -t nat -A NatACL_gdnat_%lu -s %s  -j RETURN ", (unsigned long) s_client->router_address, ntoa_address(str_client_address, htonl(s_client->client_address)));
        command_add(s_db, TYPE_IPTABLES_COMMAND, "nat", firewall_rule, 1);


    if (s_client->s_client_policy) {
        handle_policy(s_db, s_client, s_client->s_client_policy, have_natacl);
    }
    if (s_client->s_group_policy) {
        handle_policy(s_db, s_client, s_client->s_group_policy, have_natacl);
    }

    handle_policy_exec(s_db, s_client, s_client->s_client_policy, have_natacl);
    handle_policy_exec(s_db, s_client, s_client->s_group_policy, have_natacl);


    return 1;
}
int init_clients_rules(struct s_db_hdr *s_db)
{
    struct s_list_hdr *s_walker;
    struct s_client_hdr *s_client;

//    char firewall_rule[255];
//    char str_client_address[64];
//    int have_natacl = 0;

//    s_db->init = 1;

    if (s_db->s_clients) {
        s_walker = s_db->s_clients->s_first;
        while (s_walker) {
            s_client = (struct s_client_hdr *) s_walker->data;

//            command_add(s_db, TYPE_IPTABLES_LOCK, NULL, NULL, 0);     

            process_client(s_db, s_client);

            s_client->last_tv = time(NULL);
            s_client->vrfy_tv = time(NULL) + 30;

            s_walker = s_walker->s_right;
        }
    }
//    s_db->init = 0;

    return 1;
}

int check_dhcp_rule_group(struct s_db_hdr *s_db)
{
    struct s_list_hdr *s_walker;
    struct s_db_group *s_group;
    char chain[255];


    if (s_db->s_groups) {
        s_walker = s_db->s_groups->s_first;
        while (s_walker) {
            s_group = (struct s_db_group *) s_walker->data;

            snprintf(chain, 255, "NatACL_reload_%s", s_group->group_name);

            if (chain_vrfy(chain,&s_db->mutex_iptables)) {
//                snprintf(chain, 255, "NatACL_reload_%s", s_group->group_name);
                iptables_chain_delete( "filter", chain );
                command_add(s_db, TYPE_IPTABLES_REMOVE, "filter", chain, 1);

                delete_dhcp_rule_group(s_db, s_group);
            }

            s_walker = s_walker->s_right;
        }
    }
    return 1;
}


int delete_dhcp_rule_group(struct s_db_hdr *s_db, struct s_db_group *s_group)
{
    struct s_list_hdr *s_walker;

    if (s_db->s_clients) {
        s_walker = s_db->s_clients->s_first;
        while (s_walker) {
            if (((struct s_client_hdr *) s_walker->data)->bu_group_id == s_group->id) {
                db_get_address_by_mac(s_db->db, s_db, ((struct s_client_hdr *) s_walker->data)->int_mac_address);
                delete_dhcp_rule(s_db, (struct s_client_hdr *) s_walker->data);
            }
            s_walker = s_walker->s_right;
        }
    }
    return 1;
}

int delete_dhcp_rule(struct s_db_hdr *s_db, struct s_client_hdr *s_client)
{
    char firewall_rule[255];
    char str_client_address[255];

    wlog(FILE_LOG, "[INFO] [%02x:%02x:%02x:%02x:%02x:%02x] UPDATING Client - Removing ACL's >/dev/null 2>/dev/null", s_client->mac_address[0], s_client->mac_address[1], s_client->mac_address[2], s_client->mac_address[3], s_client->mac_address[4], s_client->mac_address[5]);
/* Flush user chains */
    snprintf(firewall_rule, 255, "NatACL_fwd_%02x%02x%02x%02x%02x%02x", s_client->mac_address[0], s_client->mac_address[1], s_client->mac_address[2], s_client->mac_address[3], s_client->mac_address[4], s_client->mac_address[5]);
    command_add(s_db, TYPE_IPTABLES_FLUSH, "filter", firewall_rule, 0);

    snprintf(firewall_rule, 255, "NatACL_snat_%02x%02x%02x%02x%02x%02x", s_client->mac_address[0], s_client->mac_address[1], s_client->mac_address[2], s_client->mac_address[3], s_client->mac_address[4], s_client->mac_address[5]);
    command_add(s_db, TYPE_IPTABLES_FLUSH, "nat", firewall_rule, 0);

    snprintf(firewall_rule, 255, "NatACL_dnat_%02x%02x%02x%02x%02x%02x", s_client->mac_address[0], s_client->mac_address[1], s_client->mac_address[2], s_client->mac_address[3], s_client->mac_address[4], s_client->mac_address[5]);
    command_add(s_db, TYPE_IPTABLES_FLUSH, "nat", firewall_rule, 0);

    snprintf(firewall_rule, 255, "NatACL_web_%02x%02x%02x%02x%02x%02x", s_client->mac_address[0], s_client->mac_address[1], s_client->mac_address[2], s_client->mac_address[3], s_client->mac_address[4], s_client->mac_address[5]);
    command_add(s_db, TYPE_IPTABLES_FLUSH, "nat", firewall_rule, 0);


/* dis-associate the chains */
    snprintf(firewall_rule, 255, "iptables -t nat -D NatACL_gweb_%lu -s %s -j NatACL_web_%02x%02x%02x%02x%02x%02x ", (unsigned long) s_client->router_address, ntoa_address(str_client_address, htonl(s_client->client_address)), s_client->mac_address[0], s_client->mac_address[1], s_client->mac_address[2], s_client->mac_address[3], s_client->mac_address[4], s_client->mac_address[5]);
    command_add(s_db, TYPE_IPTABLES_COMMAND, "nat", firewall_rule, 1);
    snprintf(firewall_rule, 255, "iptables -t nat -D NatACL_gweb_%lu -s %s -j RETURN ", (unsigned long) s_client->router_address, ntoa_address(str_client_address, htonl(s_client->client_address)));
    command_add(s_db, TYPE_IPTABLES_COMMAND, "nat", firewall_rule, 1);
    snprintf(firewall_rule, 255, "iptables -t nat -D NatACL_gsnat_%lu  -s %s  -j NatACL_snat_%02x%02x%02x%02x%02x%02x ", (unsigned long) s_client->router_address, ntoa_address(str_client_address, htonl(s_client->client_address)), s_client->mac_address[0], s_client->mac_address[1], s_client->mac_address[2], s_client->mac_address[3], s_client->mac_address[4], s_client->mac_address[5]);
    command_add(s_db, TYPE_IPTABLES_COMMAND, "nat", firewall_rule, 1);
    snprintf(firewall_rule, 255, "iptables -t nat -D NatACL_gsnat_%lu  -s %s  -j RETURN ", (unsigned long) s_client->router_address, ntoa_address(str_client_address, htonl(s_client->client_address)));
    command_add(s_db, TYPE_IPTABLES_COMMAND, "nat", firewall_rule, 1);
    snprintf(firewall_rule, 255, "iptables -t nat -D NatACL_gdnat_%lu -s %s  -j NatACL_dnat_%02x%02x%02x%02x%02x%02x ", (unsigned long) s_client->router_address, ntoa_address(str_client_address, htonl(s_client->client_address)), s_client->mac_address[0], s_client->mac_address[1], s_client->mac_address[2], s_client->mac_address[3], s_client->mac_address[4], s_client->mac_address[5]);
    command_add(s_db, TYPE_IPTABLES_COMMAND, "nat", firewall_rule, 1);
    snprintf(firewall_rule, 255, "iptables -t nat -D NatACL_gdnat_%lu -s %s  -j RETURN ", (unsigned long) s_client->router_address, ntoa_address(str_client_address, htonl(s_client->client_address)));
    command_add(s_db, TYPE_IPTABLES_COMMAND, "nat", firewall_rule, 1);
    snprintf(firewall_rule, 255, "iptables -D NatACL_gfwd_%lu  -p all -s %s  -j NatACL_fwd_%02x%02x%02x%02x%02x%02x", (unsigned long) s_client->router_address, ntoa_address(str_client_address, htonl(s_client->client_address)), s_client->mac_address[0], s_client->mac_address[1], s_client->mac_address[2], s_client->mac_address[3], s_client->mac_address[4], s_client->mac_address[5]);
    command_add(s_db, TYPE_IPTABLES_COMMAND, NULL, firewall_rule, 1);
    snprintf(firewall_rule, 255, "iptables -D NatACL_gfwd_%lu  -p all -s %s  -j RETURN", (unsigned long) s_client->router_address, ntoa_address(str_client_address, htonl(s_client->client_address)));
    command_add(s_db, TYPE_IPTABLES_COMMAND, NULL, firewall_rule, 1);
/* Delete users chains */
    snprintf(firewall_rule, 255, "NatACL_fwd_%02x%02x%02x%02x%02x%02x", s_client->mac_address[0], s_client->mac_address[1], s_client->mac_address[2], s_client->mac_address[3], s_client->mac_address[4], s_client->mac_address[5]);
    command_add(s_db, TYPE_IPTABLES_REMOVE, "filter", firewall_rule, 0);
    snprintf(firewall_rule, 255, "NatACL_snat_%02x%02x%02x%02x%02x%02x", s_client->mac_address[0], s_client->mac_address[1], s_client->mac_address[2], s_client->mac_address[3], s_client->mac_address[4], s_client->mac_address[5]);
    command_add(s_db, TYPE_IPTABLES_REMOVE, "nat", firewall_rule, 0);
    snprintf(firewall_rule, 255, "NatACL_dnat_%02x%02x%02x%02x%02x%02x", s_client->mac_address[0], s_client->mac_address[1], s_client->mac_address[2], s_client->mac_address[3], s_client->mac_address[4], s_client->mac_address[5]);
    command_add(s_db, TYPE_IPTABLES_REMOVE, "nat", firewall_rule, 0);
    snprintf(firewall_rule, 255, "NatACL_web_%02x%02x%02x%02x%02x%02x", s_client->mac_address[0], s_client->mac_address[1], s_client->mac_address[2], s_client->mac_address[3], s_client->mac_address[4], s_client->mac_address[5]);
    command_add(s_db, TYPE_IPTABLES_REMOVE, "nat", firewall_rule, 0);

//    default_network_group(s_db, s_client, s_db->s_groups);


    return 1;
}
int check_ip_group(struct s_db_hdr *s_db, uint32_t addr)
{
    struct s_list_hdr *s_walker;
    struct s_db_group *s_group;

    s_walker = s_db->s_groups->s_first;
    while (s_walker) {
        s_group = (struct s_db_group *) s_walker->data;

        if (ntohl(addr) > s_group->router_address && ntohl(addr) < s_group->broadcast_address) {
            return 1;
        }
        s_walker = s_walker->s_right;
    }
    return 0;
}
int init_sockets(struct s_db_hdr *s_db, int max_sockets)
{
    int on = 1;
    int i;
    int count = 0;
    struct s_list_hdr *s_walker;
    struct s_list_hdr *s_walker_iface;
    struct s_db_group *s_group;
    struct s_iface_hdr s_iface;
    struct ifreq interface;
    struct sockaddr_in saddr;
    struct s_list *s_iface_list = NULL;
    char name[255];
    int iface_found = 0;

    memset(&saddr, 0x00, sizeof(struct sockaddr_in));
//    memset(&saddr, 0x00, sizeof(struct sockaddr_in));

    saddr.sin_addr.s_addr = inet_addr("0.0.0.0");
    saddr.sin_family = AF_INET;
    saddr.sin_port = htons(67);


    for (i = 0; i < s_db->sock_fds_count; i++) {
        close(s_db->sock_fds[i]);
    }

    if (s_db->s_groups) {
        s_walker = s_db->s_groups->s_first;
        while (s_walker) {
            iface_found = 0;

            s_group = (struct s_db_group *) s_walker->data;

            if (get_iface_by_address(&s_iface, s_group->router_address)) {
                strncpy(name, s_iface.name, IFNAMSIZ);
                for (i = 0; i < IFNAMSIZ; i++) {
                    if (name[i] == ':') {
                        name[i] = 0x00;
                        break;
                    }
                    if (name[i] == 0x00) {
                        break;
                    }
                }

                if (s_iface_list) {
                    s_walker_iface = s_iface_list->s_first;
                    while (s_walker_iface) {
                        if (strcmp(s_walker_iface->data, name) == 0) {
                            iface_found = 1;
                            break;
                        }
                        s_walker_iface = s_walker_iface->s_right;
                    }
                }

                if (!iface_found) {
                    s_iface_list = list_add(s_iface_list, name, strlen(name) + 1);
                }
            }
            s_walker = s_walker->s_right;
        }
    }
    if (s_iface_list) {
        s_walker = s_iface_list->s_first;
        while (s_walker) {
            if (count < max_sockets) {
                wlog(FILE_LOG, "[INFO] LISTEN ON INTERFACE: %s", s_walker->data);

                s_db->sock_fds[count] = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);

                strncpy(interface.ifr_ifrn.ifrn_name, s_walker->data, IFNAMSIZ);
                if (setsockopt(s_db->sock_fds[count], SOL_SOCKET, SO_BINDTODEVICE, (char *) &interface, sizeof(interface)) < 0) {
                    perror("sendpacket: setting SO_BINDTODEVICE");
                    exit(1);
                }
                if (setsockopt(s_db->sock_fds[count], SOL_SOCKET, SO_BROADCAST, (char *) &on, sizeof(on)) < 0) {
                    perror("sendpacket: setting SO_BINDTODEVICE");
                    exit(1);
                }

                if (bind(s_db->sock_fds[count], (struct sockaddr *) &saddr, sizeof(struct sockaddr)) < 0) {
                    perror("bind");
                    exit(0);
                }

                count++;
            } else {
                break;
            }
            s_walker = s_walker->s_right;
        }

        list_clear(s_iface_list);
    }
    s_db->sock_fds_count = count;

    return count;
}
int command_add(struct s_db_hdr *s_db, int type, char *tablename, char *command, int verify)
{
    struct s_exec_hdr s_exec;

    snprintf(s_exec.command, 4096, "%s", command);
    if (tablename)
        snprintf(s_exec.table, 4096, "%s", tablename);
    s_exec.verify = verify;
    s_exec.type = type;

    pthread_mutex_lock(&s_db->mutex_queue_command);
    s_db->s_queue_command = list_add(s_db->s_queue_command, (char *) &s_exec, sizeof(struct s_exec_hdr));
    pthread_mutex_unlock(&s_db->mutex_queue_command);

    return 1;
}

void *command_handler(void *data)
{
    struct s_db_hdr *s_db = (struct s_db_hdr *) data;
//    struct s_list_hdr * s_walker;
    struct s_exec_hdr s_exec;
    int status;
    int process = 0;
    int lock_tv=0;
    while (1) {
    	lock_tv = time(NULL);
    	
        while (check_lock()) {
            wlog(FILE_LOG, "[WARNING] Lock file detected - sleeping 1s");
            if (time(NULL) > lock_tv + 30) {
                wlog(FILE_LOG, "[WARNING] Lock file is > 30s old, removing.");
                remove(FILENAME_LOCK);
            }
            sleep(1);
        }
        process = 0;
        if (s_db->s_queue_command) {
            pthread_mutex_lock(&s_db->mutex_queue_command);
            if (s_db->s_queue_command->s_first) {
                memcpy(&s_exec, s_db->s_queue_command->s_first->data, sizeof(struct s_exec_hdr));
                process = 1;

                list_del(s_db->s_queue_command, s_db->s_queue_command->s_first);
            }
            pthread_mutex_unlock(&s_db->mutex_queue_command);

            if (process) {
                switch (s_exec.type) {
                case TYPE_IPTABLES_COMMAND:
        	    pthread_mutex_lock(&s_db->mutex_iptables);
                    if (s_exec.verify) {
                        if ((status = system(s_exec.command)) > 0) {
                            wlog(FILE_LOG, "COMMAND FAILURE [%d]: %s", status, s_exec.command);
                        }
                    } else {
                        system(s_exec.command);
                    }
	            pthread_mutex_unlock(&s_db->mutex_iptables);

                    break;
                case TYPE_IPTABLES_FLUSH:
        	    pthread_mutex_lock(&s_db->mutex_iptables);
                    wlog(FILE_LOG, "[INIT] Flushing chain: %s/%s", s_exec.table,s_exec.command );
                    iptables_chain_flush(s_exec.table, s_exec.command);
        	    pthread_mutex_unlock(&s_db->mutex_iptables);
                    break;
                case TYPE_IPTABLES_REMOVE:
        	    pthread_mutex_lock(&s_db->mutex_iptables);
                    wlog(FILE_LOG, "[INIT] Removing chain: %s/%s", s_exec.table,s_exec.command );
                    iptables_chain_delete(s_exec.table, s_exec.command);
        	    pthread_mutex_unlock(&s_db->mutex_iptables);
                    break;
                case TYPE_IPTABLES_CREATE:
        	    pthread_mutex_lock(&s_db->mutex_iptables);
                    iptables_chain_create(s_exec.table, s_exec.command);
        	    pthread_mutex_unlock(&s_db->mutex_iptables);
                    break;
                case TYPE_IPTABLES_UNLOCK:
                    s_db->init = 0;
                    wlog(FILE_LOG, "[INFO] IPTABLES INIT OK");
                    break;
                }
            } else {
                usleep(100000);
            }
        } else {
            usleep(100000);
        }
    }
    return NULL;
}
void init_server(int option_auto_add)
{
    int i;
    int lock_tv;
    int size;
    int have_natacl = 0;
    char packet[1500];
    socklen_t slen = sizeof(struct sockaddr);
    int group_check = 0;
    int packet_ok;
    int fwd_check = 0;


    struct s_dhcp_hdr s_dhcp;
    struct sockaddr_in sin;
    struct in_addr in;
    uint64_t *v;
    char str_client_address[255];
    char chain[255];
//    char str_router_address[255];
    int flag_individual = 0;
    uint32_t client_address;
    int reload = 0;
    struct s_client_hdr *s_client;
    struct s_db_group s_group;
    int expire_check_tv = 0;
    pthread_attr_t attr_command;
    pthread_t thread_command;

    fd_set fds;
    struct timeval tv;


    create_pidfile(getpid());

    memset(&s_db, 0x00, sizeof(struct s_db_hdr));

    s_db.db = init_db();

//    sock = dhcp_bind();
//      init_sockets( &s_db, sock_fds, &sock_fds_count, 65535 );

    pthread_attr_init(&attr_command);

    memset(&sin, 0x00, sizeof(struct sockaddr_in));
    memset(&s_dhcp, 0x00, sizeof(struct s_dhcp_hdr));


    if (!get_group_by_name(s_db.db, "unknown", &s_group)) {
        printf("ERROR: There is no group called 'unknown', This group is required. create it using \"NatACL_config group add\"\n");
        exit(0);
    }


    db_get_group(s_db.db, &s_db);
    db_get_address(s_db.db, &s_db);

    if (s_db.s_clients) {
        printf("Total MAC ADDRESS in database: %d\n", s_db.s_clients->count);
        fflush(stdout);
    } else {
        printf("NO CLIENT MAC\n");
        fflush(stdout);
    }
    wlog(FILE_LOG, "[INFO] AUTO ADD: %d", option_auto_add);

    s_db.init = 1;
    pthread_create(&thread_command, &attr_command, command_handler, (void *) &s_db);



    init_rules(&s_db, s_db.s_clients, s_db.s_groups);
    command_add(&s_db, TYPE_IPTABLES_UNLOCK, NULL, NULL, 0);

//    init_clients_rules(&s_db);


    while (1) {
        reload = 0;
        lock_tv = time(NULL);
        packet_ok = 0;

        if (lock_tv > group_check + 15) {

            check_dhcp_rule_group(&s_db);

            if (!s_db.sock_fds_flag) {
                wlog(FILE_LOG, "[INFO] Init Sockets");

                init_sockets(&s_db, 65535);

                s_db.sock_fds_flag = 1;
            }
            group_check = lock_tv;
        }
        FD_ZERO(&fds);
        for (i = 0; i < s_db.sock_fds_count; i++) {
            FD_SET(s_db.sock_fds[i], &fds);
        }

        if (!s_db.init && !chain_vrfy("NatACL_fwd",&s_db.mutex_iptables) && lock_tv > fwd_check + 5) {
            command_add(&s_db, TYPE_IPTABLES_CREATE, "filter", "NatACL_LOGDROP", 0);
            command_add(&s_db, TYPE_IPTABLES_CREATE, "filter", "NatACL_fwd", 0);

            command_add(&s_db, TYPE_IPTABLES_CREATE, "nat", "NatACL_snat", 0);
            command_add(&s_db, TYPE_IPTABLES_CREATE, "nat", "NatACL_dnat", 0);
            command_add(&s_db, TYPE_IPTABLES_CREATE, "nat", "NatACL_web", 0);

            command_add(&s_db, TYPE_IPTABLES_COMMAND, NULL, "iptables -A NatACL_LOGDROP -j LOG", 0);
            command_add(&s_db, TYPE_IPTABLES_COMMAND, NULL, "iptables -A NatACL_LOGDROP -j DROP", 0);
            command_add(&s_db, TYPE_IPTABLES_COMMAND, NULL, "iptables -I FORWARD -j NatACL_fwd", 0);
            command_add(&s_db, TYPE_IPTABLES_COMMAND, NULL, "iptables -t nat -I POSTROUTING -j NatACL_snat", 0);
            command_add(&s_db, TYPE_IPTABLES_COMMAND, NULL, "iptables -t nat -I PREROUTING -j NatACL_dnat", 0);
            command_add(&s_db, TYPE_IPTABLES_COMMAND, NULL, "iptables -t nat -I PREROUTING -j NatACL_web", 0);

            fwd_check = lock_tv;
        }

        if (lock_tv > expire_check_tv + 5) {
            expire_clients(&s_db);

            expire_check_tv = lock_tv;
        }






        tv.tv_sec = 1;
        tv.tv_usec = 0;


        select(s_db.sock_fds[s_db.sock_fds_count - 1] + 1, &fds, NULL, NULL, &tv);

        for (i = 0; i < s_db.sock_fds_count; i++) {
            if (FD_ISSET(s_db.sock_fds[i], &fds)) {
                sin.sin_addr.s_addr = inet_addr("0.0.0.0");
                sin.sin_family = AF_INET;
                sin.sin_port = htons(67);


                have_natacl = 0;
                flag_individual = 0;
                memset(&s_dhcp, 0x00, sizeof(struct s_dhcp_hdr));



                size = recvfrom(s_db.sock_fds[i], packet, 1500, 0, (struct sockaddr *) &sin, &slen);

                if (size > 0 && dhcp_decode_packet(&s_dhcp, packet, size)) {
                    v = (uint64_t *) s_dhcp.address;

                    if (*v) {
                        /* If MAC ADDRESS exists on database */
                        /* Check if chain exist, if exist, reload MAC address info from database */
                        snprintf(chain, 255, "NatACL_reload_%02x%02x%02x%02x%02x%02x", s_dhcp.address[0], s_dhcp.address[1], s_dhcp.address[2], s_dhcp.address[3], s_dhcp.address[4], s_dhcp.address[5]);
                        if (chain_vrfy(chain,&s_db.mutex_iptables)) {
                            if ((s_client = search_client_by_mac(s_db.s_clients, *v))) {
                                delete_dhcp_rule(&s_db, s_client);
                            }

                            wlog(FILE_LOG, "[%02x:%02x:%02x:%02x:%02x:%02x] [UPDATE REQUEST]", s_dhcp.address[0], s_dhcp.address[1], s_dhcp.address[2], s_dhcp.address[3], s_dhcp.address[4], s_dhcp.address[5]);
                            if (s_client->s_client_policy)
                                list_clear(s_client->s_client_policy);
                            if (s_client->s_group_policy)
                                list_clear(s_client->s_group_policy);

                            s_client->s_client_policy = NULL;
                            s_client->s_group_policy = NULL;

                            db_get_address_by_mac(s_db.db, &s_db, *v);

                            command_add(&s_db, TYPE_IPTABLES_REMOVE, "filter", chain, 0);
                        }

                        if ((s_client = search_client_by_mac(s_db.s_clients, *v))) {
                            s_dhcp.response_broadcast_address = htonl(s_client->broadcast_address);
                            s_dhcp.response_client_address = htonl(s_client->client_address);
                            s_dhcp.response_dhcp_address = htonl(s_client->router_address);
                            s_dhcp.response_dns_address = htonl(s_client->dns_address);
                            s_dhcp.response_router_address = htonl(s_client->router_address);
                            s_dhcp.response_wins_address = htonl(s_client->wins_address);
                            snprintf(s_dhcp.response_domain, 255, s_client->domain);
                            s_client->last_tv = time(NULL);
                        } else {
                            if (option_auto_add == 1) {
                                if ((client_address = get_free_address(s_db.db, &s_db, "unknown"))) {
                                    client_add(s_db.db, *v, "unknown", client_address, "Undefined");

                                    db_get_address_by_mac(s_db.db, &s_db, *v);

                                    if ((s_client = search_client_by_mac(s_db.s_clients, *v))) {
                                        s_dhcp.response_broadcast_address = htonl(s_client->broadcast_address);
                                        s_dhcp.response_client_address = htonl(s_client->client_address);
                                        s_dhcp.response_dhcp_address = htonl(s_client->router_address);
                                        s_dhcp.response_dns_address = htonl(s_client->dns_address);
                                        s_dhcp.response_router_address = htonl(s_client->router_address);
                                        s_dhcp.response_wins_address = htonl(s_client->wins_address);

                                        snprintf(s_dhcp.response_domain, 255, s_client->domain);
                                        s_client->last_tv = time(NULL);

                                    } else {
                                        printf("ERROR - CLIENT MAC NOT FOUND - ?!?!?!\n");
                                        fflush(stdout);
                                    }
                                    wlog(FILE_LOG, "[%02x:%02x:%02x:%02x:%02x:%02x] New client found - Adding to group [unknown]", s_dhcp.address[0], s_dhcp.address[1], s_dhcp.address[2], s_dhcp.address[3], s_dhcp.address[4], s_dhcp.address[5]);
                                } else {
                                    wlog(FILE_LOG, "[%02x:%02x:%02x:%02x:%02x:%02x] New client found - ERROR - OUT OF IP ADDRESS", s_dhcp.address[0], s_dhcp.address[1], s_dhcp.address[2], s_dhcp.address[3], s_dhcp.address[4], s_dhcp.address[5]);
                                }
                            } else {
                                in.s_addr = s_dhcp.requested_client_address;
                                wlog(FILE_LOG, "[%02x:%02x:%02x:%02x:%02x:%02x] [SECURITY] DHCP REQUEST DENIED", s_dhcp.address[0], s_dhcp.address[1], s_dhcp.address[2], s_dhcp.address[3], s_dhcp.address[4], s_dhcp.address[5]);
                            }
                        }
                        /* time check to avoid flood and problem with slow computers */

                        if (s_client) {
                            s_client->vrfy_tv = time(NULL);

                            s_dhcp.source_address = sin.sin_addr.s_addr;

                            switch (s_dhcp.type) {
                            case DHCP_TYPE_DISCOVER:
                                wlog(FILE_LOG, "[%02x:%02x:%02x:%02x:%02x:%02x] [DISCOVER]\n", s_dhcp.address[0], s_dhcp.address[1], s_dhcp.address[2], s_dhcp.address[3], s_dhcp.address[4], s_dhcp.address[5]);
                                dhcp_send_response("", &s_dhcp);
                                break;
                            case DHCP_TYPE_REQUEST:
                                wlog(FILE_LOG, "[%02x:%02x:%02x:%02x:%02x:%02x] [REQUEST] %s", s_dhcp.address[0], s_dhcp.address[1], s_dhcp.address[2], s_dhcp.address[3], s_dhcp.address[4], s_dhcp.address[5], ntoa_address(str_client_address, s_dhcp.requested_client_address));

                                if (s_dhcp.requested_client_address == htonl(s_client->client_address)) {
                                    dhcp_send_response("", &s_dhcp);

                                    snprintf(chain, 255, "NatACL_fwd_%02x%02x%02x%02x%02x%02x", s_dhcp.address[0], s_dhcp.address[1], s_dhcp.address[2], s_dhcp.address[3], s_dhcp.address[4], s_dhcp.address[5]);

                                    if (!s_db.init) {
                                        if (!chain_vrfy(chain,&s_db.mutex_iptables)) {
			                                wlog(FILE_LOG, "[%02x:%02x:%02x:%02x:%02x:%02x] [%s] Adding firewall rules", s_dhcp.address[0], s_dhcp.address[1], s_dhcp.address[2], s_dhcp.address[3], s_dhcp.address[4], s_dhcp.address[5], ntoa_address(str_client_address, s_dhcp.requested_client_address));

                                            process_client(&s_db, s_client);
                                        } else {
			                                wlog(FILE_LOG, "[DEBUG] CLIENT IS OK [%s]", chain);

                                            handle_policy_exec(&s_db, s_client, s_client->s_client_policy, have_natacl);
                                            handle_policy_exec(&s_db, s_client, s_client->s_group_policy, have_natacl);
                                        }
                                    } else {
                                        wlog(FILE_LOG, "[%02x:%02x:%02x:%02x:%02x:%02x] [INFO] [INIT FASE - IPTABLES IN WAIT FASE]\n", s_dhcp.address[0], s_dhcp.address[1], s_dhcp.address[2], s_dhcp.address[3], s_dhcp.address[4], s_dhcp.address[5]);
                                    }

                                } else {
                                    dhcp_send_response_nak("", &s_dhcp);


                                    in.s_addr = s_dhcp.requested_client_address;
                                    wlog(FILE_LOG, "[%02x:%02x:%02x:%02x:%02x:%02x] [DHCP REQUEST DENIED] CLIENT ADDRESS [%s]", s_dhcp.address[0], s_dhcp.address[1], s_dhcp.address[2], s_dhcp.address[3], s_dhcp.address[4], s_dhcp.address[5], inet_ntoa(in));
                                    in.s_addr = htonl(s_client->client_address);
                                    wlog(FILE_LOG, "[%02x:%02x:%02x:%02x:%02x:%02x] [DHCP REQUEST DENIED] CLIENT ADDRESS [%s]", s_dhcp.address[0], s_dhcp.address[1], s_dhcp.address[2], s_dhcp.address[3], s_dhcp.address[4], s_dhcp.address[5], inet_ntoa(in));
                                }
	                            break;
                            }
                        }
                    }
                }
            }
        }
    }
}
int expire_clients(struct s_db_hdr *s_db)
{
    struct s_client_hdr *s_client;
    struct s_list_hdr *s_walker;
    int now;
//    char firewall_rule[255];

//    char str_client_address[255];
    now = time(NULL);

    if (s_db->s_clients) {
        s_walker = s_db->s_clients->s_first;
        while (s_walker) {
            s_client = (struct s_client_hdr *) s_walker->data;
            if (s_client->last_tv && now > s_client->last_tv + 3600) {
                s_client->last_tv = 0;

                wlog(FILE_LOG, "[INFO] [%02x:%02x:%02x:%02x:%02x:%02x] Expired Client - Removing ACL's >/dev/null 2>/dev/null", s_client->mac_address[0], s_client->mac_address[1], s_client->mac_address[2], s_client->mac_address[3], s_client->mac_address[4], s_client->mac_address[5]);

                delete_dhcp_rule(s_db, s_client);
            }
            s_walker = s_walker->s_right;
        }
    }
    return 1;
}
int default_network_group(struct s_db_hdr *s_db, struct s_client_hdr *s_client, struct s_list *s_l)
{
    char firewall_rule[512];
    char str_client_address[255];
    char str_router_address[255];
    char str_mask_address[255];

    struct s_list_hdr *s_walker;
    struct s_db_group *s_group;

    if (s_l) {
        s_walker = s_l->s_first;
        while (s_walker) {
            s_group = (struct s_db_group *) s_walker->data;

            if (s_group->router_address != s_client->router_address) {
                snprintf(firewall_rule, 255, "iptables -D NatACL_fwd_%02x%02x%02x%02x%02x%02x -p all -s %s -d %s/%s -j NatACL_LOGDROP", s_client->mac_address[0], s_client->mac_address[1], s_client->mac_address[2], s_client->mac_address[3], s_client->mac_address[4], s_client->mac_address[5], ntoa_address(str_client_address, htonl(s_client->client_address)), ntoa_address(str_router_address, htonl(s_group->router_address)), ntoa_address(str_mask_address, ntohl(s_group->netmask_address)));
                command_add(s_db, TYPE_IPTABLES_COMMAND, "nat", firewall_rule, 0);

                snprintf(firewall_rule, 255, "iptables -A NatACL_fwd_%02x%02x%02x%02x%02x%02x -p all -s %s -d %s/%s -j NatACL_LOGDROP", s_client->mac_address[0], s_client->mac_address[1], s_client->mac_address[2], s_client->mac_address[3], s_client->mac_address[4], s_client->mac_address[5], ntoa_address(str_client_address, htonl(s_client->client_address)), ntoa_address(str_router_address, htonl(s_group->router_address)), ntoa_address(str_mask_address, ntohl(s_group->netmask_address)));
                command_add(s_db, TYPE_IPTABLES_COMMAND, "nat", firewall_rule, 1);

            }

            s_walker = s_walker->s_right;
        }
    }





    return 1;
}

int handle_policy(struct s_db_hdr *s_db, struct s_client_hdr *s_client, struct s_list *s_l, int check_natacl)
{
    char str_client_address[64];
//    char str_router_address[64];
    char str_dnat_address[64];
    char str_snat_address[64];
    char str_router_address[64];
    char str_mask_address[64];
    char firewall_rule[255];
    struct s_list_hdr *s_walker;
    struct s_db_policy *s_policy;
    struct s_iface_hdr s_iface;
    struct s_db_group *s_group;

    if (s_l) {
        s_walker = s_l->s_first;
        while (s_walker) {
            s_policy = (struct s_db_policy *) s_walker->data;
            if (check_natacl && s_policy->type == TYPE_NATACL) {
                wlog(FILE_LOG, "[%02x:%02x:%02x:%02x:%02x:%02x] [NATACL GROUP FOUND- RUNNING IPTABLES]", s_client->mac_address[0], s_client->mac_address[1], s_client->mac_address[2], s_client->mac_address[3], s_client->mac_address[4], s_client->mac_address[5]);

                snprintf(firewall_rule, 255, "iptables -t nat -A NatACL_web_%02x%02x%02x%02x%02x%02x -p tcp  -d 0/0 --dport 80 -j REDIRECT --to-port 5121", s_client->mac_address[0], s_client->mac_address[1], s_client->mac_address[2], s_client->mac_address[3], s_client->mac_address[4], s_client->mac_address[5]);
                command_add(s_db, TYPE_IPTABLES_COMMAND, "nat", firewall_rule, 1);

                snprintf(firewall_rule, 255, "iptables -t nat -A NatACL_gweb_%lu -s %s -j NatACL_web_%02x%02x%02x%02x%02x%02x ", (unsigned long) s_client->router_address, ntoa_address(str_client_address, htonl(s_client->client_address)), s_client->mac_address[0], s_client->mac_address[1], s_client->mac_address[2], s_client->mac_address[3], s_client->mac_address[4], s_client->mac_address[5]);
                command_add(s_db, TYPE_IPTABLES_COMMAND, "nat", firewall_rule, 1);


                snprintf(firewall_rule, 255, "iptables -t nat -A NatACL_gweb_%lu -s %s -j RETURN ", (unsigned long) s_client->router_address, ntoa_address(str_client_address, htonl(s_client->client_address)));
                command_add(s_db, TYPE_IPTABLES_COMMAND, "nat", firewall_rule, 1);

            }

            if (!check_natacl && s_policy->type == TYPE_SNAT) {
                if (get_iface_by_address(&s_iface, ntohl(s_policy->snat_address))) {
                    wlog(FILE_LOG, "[%02x:%02x:%02x:%02x:%02x:%02x] [SNAT GROUP FOUND- RUNNING IPTABLES]", s_client->mac_address[0], s_client->mac_address[1], s_client->mac_address[2], s_client->mac_address[3], s_client->mac_address[4], s_client->mac_address[5]);

                    snprintf(firewall_rule, 255, "iptables -t nat -o %s  -A NatACL_snat_%02x%02x%02x%02x%02x%02x  -p all  -j SNAT --to-source %s", s_iface.name, s_client->mac_address[0], s_client->mac_address[1], s_client->mac_address[2], s_client->mac_address[3], s_client->mac_address[4], s_client->mac_address[5], ntoa_address(str_snat_address, s_policy->snat_address));
                    command_add(s_db, TYPE_IPTABLES_COMMAND, "nat", firewall_rule, 1);
/*                    snprintf(firewall_rule, 255, "iptables -t nat -A NatACL_gsnat_%lu  -s %s  -j NatACL_snat_%02x%02x%02x%02x%02x%02x ", (unsigned long) s_client->router_address, ntoa_address(str_client_address, htonl(s_client->client_address)), s_client->mac_address[0], s_client->mac_address[1], s_client->mac_address[2], s_client->mac_address[3], s_client->mac_address[4], s_client->mac_address[5]);
                    command_add(s_db, TYPE_IPTABLES_COMMAND, "nat", firewall_rule, 1);
                    snprintf(firewall_rule, 255, "iptables -t nat -A NatACL_gsnat_%lu  -s %s  -j RETURN ", (unsigned long) s_client->router_address, ntoa_address(str_client_address, htonl(s_client->client_address)));
                    command_add(s_db, TYPE_IPTABLES_COMMAND, "nat", firewall_rule, 1);
*/
                }
            }
            if (!check_natacl && s_policy->type == TYPE_DNAT) {
                wlog(FILE_LOG, "[%02x:%02x:%02x:%02x:%02x:%02x] [DNAT GROUP FOUND- RUNNING IPTABLES]", s_client->mac_address[0], s_client->mac_address[1], s_client->mac_address[2], s_client->mac_address[3], s_client->mac_address[4], s_client->mac_address[5]);

                snprintf(firewall_rule, 255, "iptables -t nat -A NatACL_dnat_%02x%02x%02x%02x%02x%02x  -p tcp -d 0/0 --dport 80 -j DNAT --to-destination %s:%d", s_client->mac_address[0], s_client->mac_address[1], s_client->mac_address[2], s_client->mac_address[3], s_client->mac_address[4], s_client->mac_address[5], ntoa_address(str_dnat_address, s_policy->dnat_address), s_policy->dnat_port);
                command_add(s_db, TYPE_IPTABLES_COMMAND, "nat", firewall_rule, 1);
/*
                snprintf(firewall_rule, 255, "iptables -t nat -A NatACL_gdnat_%lu -s %s  -j NatACL_dnat_%02x%02x%02x%02x%02x%02x ", (unsigned long) s_client->router_address, ntoa_address(str_client_address, htonl(s_client->client_address)), s_client->mac_address[0], s_client->mac_address[1], s_client->mac_address[2], s_client->mac_address[3], s_client->mac_address[4], s_client->mac_address[5]);
                command_add(s_db, TYPE_IPTABLES_COMMAND, "nat", firewall_rule, 1);

                snprintf(firewall_rule, 255, "iptables -t nat -A NatACL_gdnat_%lu -s %s  -j RETURN ", (unsigned long) s_client->router_address, ntoa_address(str_client_address, htonl(s_client->client_address)));
                command_add(s_db, TYPE_IPTABLES_COMMAND, "nat", firewall_rule, 1);
*/
            }

            if (!check_natacl && s_policy->type == TYPE_GROUP) {
                wlog(FILE_LOG, "[%02x:%02x:%02x:%02x:%02x:%02x] [NETWORK GROUP ACCESS FOUND - ACCESS TO GROUP %s GRANTED]", s_client->mac_address[0], s_client->mac_address[1], s_client->mac_address[2], s_client->mac_address[3], s_client->mac_address[4], s_client->mac_address[5], s_policy->config);

                memset(&s_group, 0x00, sizeof(struct s_db_group));

                if ((s_group = cache_get_group_by_name(s_db->s_groups, s_policy->config))) {
//                          if (get_group_by_name(s_db.db, s_policy->config, &s_group)) {
                    snprintf(firewall_rule, 255, "iptables -D NatACL_fwd_%02x%02x%02x%02x%02x%02x  -p all -s %s -d %s/%s -j NatACL_LOGDROP", s_client->mac_address[0], s_client->mac_address[1], s_client->mac_address[2], s_client->mac_address[3], s_client->mac_address[4], s_client->mac_address[5], ntoa_address(str_client_address, htonl(s_client->client_address)), ntoa_address(str_router_address, htonl(s_group->router_address)), ntoa_address(str_mask_address, htonl(s_group->netmask_address)));
                    command_add(s_db, TYPE_IPTABLES_COMMAND, "nat", firewall_rule, 1);
//                          wlog(FILE_LOG,"%s\n",firewall_rule);
                }
            }
            s_walker = s_walker->s_right;
        }
    }
    return 1;
}
int handle_policy_exec(struct s_db_hdr *s_db, struct s_client_hdr *s_client, struct s_list *s_l, int check_natacl)
{
    struct s_list_hdr *s_walker;
//    int have_natacl = 0;
    struct s_db_policy *s_policy;
    char command[255];
    char option[255];

    struct in_addr in;

    if (!check_natacl) {
        if (s_l) {
            s_walker = s_l->s_first;
            while (s_walker) {
                s_policy = (struct s_db_policy *) s_walker->data;

                if (time(NULL) > s_policy->exec_tv + 300) {
                    s_policy->exec_tv = time(NULL);

                    if (s_policy->type == TYPE_EXEC) {
                        if (get_option("ONCE", s_policy->config, option, 255)) {
                            if (strcmp(option, "1") == 0) {
                                s_policy->once = 1;
                            } else {
                                s_policy->once = 0;
                            }
                        } else {
                            s_policy->once = 0;
                        }
                        if ((s_policy->once && !s_policy->executed) || !s_policy->once) {

                            if (get_option("COMMAND", s_policy->config, option, 255)) {
                                wlog(FILE_LOG, "[%02x:%02x:%02x:%02x:%02x:%02x] ONCE: %d [EXEC %s]", s_client->mac_address[0], s_client->mac_address[1], s_client->mac_address[2], s_client->mac_address[3], s_client->mac_address[4], s_client->mac_address[5], s_policy->once, s_policy->config);

                                in.s_addr = ntohl(s_client->client_address);

                                snprintf(command, 255, "%s %s %02x%02x%02x%02x%02x%02x >/dev/null 2>/dev/null", option, inet_ntoa(in), s_client->mac_address[0], s_client->mac_address[1], s_client->mac_address[2], s_client->mac_address[3], s_client->mac_address[4], s_client->mac_address[5]);

                                command_add(s_db, TYPE_IPTABLES_COMMAND, "nat", command, 0);

                                s_policy->executed = 1;
                            }
                        }
                    }
                }
                s_walker = s_walker->s_right;
            }
        }
    }
    return 1;
}
void reload(int sg)
{
    wlog(FILE_LOG, "[INFO] RELOAD");


    wlog(FILE_LOG, "[INFO] Cleaning cache");
    free_memory(&s_db);

    wlog(FILE_LOG, "[INFO] Loading dabatase in memory");
    db_get_group(s_db.db, &s_db);
    db_get_address(s_db.db, &s_db);

    wlog(FILE_LOG, "[INFO] Running default rules");

    init_rules(&s_db, s_db.s_clients, s_db.s_groups);
    init_sockets(&s_db, 65535);

}
int main(int argc, char **argv)
{

    signal(SIGUSR1, reload);


    int i;
    int auto_add = 1;


    if (argc > 1) {
        if (strcasecmp(argv[1], "NOAUTO") == 0) {
            auto_add = 0;
        }
    }
    i = 0;
    if ((i = fork()) > 0) {

        return 0;
    } else if (i < 0) {
        exit(1);
    }

    setsid();
    for (i = getdtablesize(); i >= 0; --i)
        close(i);
    i = open("/dev/null", O_RDWR);
    dup(i);
    dup(i);

    init_server(auto_add);
    return 1;

}

int check_lock()
{

    if (get_file_size(FILENAME_LOCK) != -1) {
        return 1;

    }
    return 0;

}
int client_add(sqlite3 * db, uint64_t mac_address, char *str, uint32_t client_address, char *name)
{
    struct s_db_group s_group;

    if (!get_group_by_name(db, str, &s_group)) {
        printf("ERROR: There is no group called 'unknown', This group is required. create it using \"NatACL_config group add\"\n");
        exit(0);
    }


    db_insert_client(db, &s_group, mac_address, name, client_address);
    return 1;
}

uint32_t get_free_address(sqlite3 * db, struct s_db_hdr * s_db, char *group_name)
{
    struct s_db_group s_group;
    uint32_t i;


    if (!get_group_by_name(db, group_name, &s_group)) {
        printf("ERROR: There is no group called 'unknown', This group is required. create it using \"NatACL_config group add\"\n");
        exit(0);
    }

    for (i = s_group.router_address + 10; i < s_group.broadcast_address; i++) {
        if (!search_client_by_ip(s_db->s_clients, i)) {
            return i;
        }
    }
    return 0;
}
int create_pidfile(int pid)
{
    FILE *fp;

    if (!(fp = fopen("/var/NatACL/NatACL_dhcp.pid", "w+"))) {
        wlog(FILE_LOG, "[ERROR] cannot create file /var/NatACL/NatACL_dhcp.pid");

        perror("/var/NatACL/NatACL_dhcp.pid");
        exit(0);
    }
    fprintf(fp, "%d", pid);
    fflush(fp);
    fclose(fp);

    return 1;
}
